@@ -4,6 +4,7 @@
  * LICENSE-TIMESCALE for a copy of the license.
  */
 #include <postgres.h>
+#include "extension_constants.h"
 #include <access/htup.h>
 #include <access/htup_details.h>
 #include <access/multixact.h>
@@ -11,6 +12,7 @@
 #include <access/table.h>
 #include <access/tableam.h>
 #include <access/transam.h>
+#include <access/tupconvert.h>
 #include <access/xact.h>
 #include <catalog/catalog.h>
 #include <catalog/dependency.h>
@@ -20,14 +22,15 @@
 #include <catalog/pg_am.h>
 #include <catalog/pg_class.h>
 #include <catalog/pg_constraint.h>
-#include <catalog/pg_constraint_d.h>
 #include <catalog/pg_foreign_server.h>
 #include <catalog/pg_foreign_table.h>
+#include <catalog/pg_type.h>
 #include <commands/defrem.h>
 #include <commands/tablecmds.h>
 #include <commands/vacuum.h>
 #include <common/relpath.h>
 #include <executor/executor.h>
+#include <executor/tuptable.h>
 #include <fmgr.h>
 #include <foreign/foreign.h>
 #include <funcapi.h>
@@ -55,6 +58,9 @@
 #include <utils/syscache.h>
 #include <utils/tuplestore.h>
 
+#include "compat/compat.h"
+#include <string.h>
+#include "annotations.h"
 #include "cache.h"
 #include "chunk.h"
 #include "debug_point.h"
@@ -62,6 +68,8 @@
 #include "hypercube.h"
 #include "hypertable.h"
 #include "hypertable_cache.h"
+#include "ts_catalog/catalog.h"
+#include "ts_catalog/compression_chunk_size.h"
 #include "utils.h"
 
 /* Data in a frozen chunk cannot be modified. So any operation
@@ -212,6 +220,7 @@ typedef struct RelationMergeInfo
 {
 	Oid relid;
 	struct VacuumCutoffs cutoffs;
+	FormData_compression_chunk_size ccs;
 	Chunk *chunk;
 	Relation rel;
 	char relpersistence;
@@ -576,7 +585,7 @@ chunk_update_constraints(const Chunk *chunk, const Hypercube *new_cube)
 		table_close(rel, NoLock);
 	}
 
-	ts_cache_release(hcache);
+	ts_cache_release(&hcache);
 }
 
 static void
@@ -625,6 +634,24 @@ merge_chunks_lock_upgrade_mode(void)
 	return MERGE_LOCK_ACCESS_EXCLUSIVE;
 }
 
+/*
+ * Use anonymous settings value to disable multidim merges due to a bug in the
+ * routing cache with non-aligned partitions/chunks.
+ */
+static bool
+merge_chunks_multidim_allowed(void)
+{
+	const char *multidim_merge_enabled =
+		GetConfigOption(MAKE_EXTOPTION("enable_merge_multidim_chunks"), true, false);
+
+	if (multidim_merge_enabled == NULL)
+		return false;
+
+	return (pg_strcasecmp("on", multidim_merge_enabled) == 0 ||
+			pg_strcasecmp("1", multidim_merge_enabled) == 0 ||
+			pg_strcasecmp("true", multidim_merge_enabled) == 0);
+}
+
 #if (PG_VERSION_NUM >= 170000 && PG_VERSION_NUM <= 170002)
 /*
  * Workaround for changed behavior in the relation rewrite code that appeared
@@ -816,13 +843,16 @@ merge_relinfos(RelationMergeInfo *relinfos, int nrelids, int mergeindex)
 	struct VacuumCutoffs *merged_cutoffs = &result_minfo->cutoffs;
 
 	/* Create the transient heap that will receive the re-ordered data */
-	Oid new_relid = make_new_heap_compat(RelationGetRelid(result_rel),
-										 tablespace,
-										 result_rel->rd_rel->relam,
-										 result_minfo->relpersistence,
-										 ExclusiveLock);
+	Oid new_relid = make_new_heap(RelationGetRelid(result_rel),
+								  tablespace,
+								  result_rel->rd_rel->relam,
+								  result_minfo->relpersistence,
+								  ExclusiveLock);
 	Relation new_rel = table_open(new_relid, AccessExclusiveLock);
 	double total_num_tuples = 0.0;
+	FormData_compression_chunk_size merged_ccs;
+
+	memset(&merged_ccs, 0, sizeof(FormData_compression_chunk_size));
 
 	pg17_workaround_init(new_rel, relinfos, nrelids);
 
@@ -839,6 +869,26 @@ merge_relinfos(RelationMergeInfo *relinfos, int nrelids, int mergeindex)
 			total_num_tuples += num_tuples;
 			relinfo->rel = NULL;
 		}
+
+		/*
+		 * Merge compression chunk size stats.
+		 *
+		 * Simply sum up the stats for all compressed relations that are
+		 * merged. Note that we don't add anything for non-compressed
+		 * relations that are merged because they don't have stats. This is a
+		 * bit weird because the data from uncompressed relations will not be
+		 * reflected in the stats of the merged chunk although the data is
+		 * part of the chunk.
+		 */
+		merged_ccs.compressed_heap_size += relinfo->ccs.compressed_heap_size;
+		merged_ccs.compressed_toast_size += relinfo->ccs.compressed_toast_size;
+		merged_ccs.compressed_index_size += relinfo->ccs.compressed_index_size;
+		merged_ccs.uncompressed_heap_size += relinfo->ccs.uncompressed_heap_size;
+		merged_ccs.uncompressed_toast_size += relinfo->ccs.uncompressed_toast_size;
+		merged_ccs.uncompressed_index_size += relinfo->ccs.uncompressed_index_size;
+		merged_ccs.numrows_post_compression += relinfo->ccs.numrows_post_compression;
+		merged_ccs.numrows_pre_compression += relinfo->ccs.numrows_pre_compression;
+		merged_ccs.numrows_frozen_immediately += relinfo->ccs.numrows_frozen_immediately;
 	}
 
 	pg17_workaround_cleanup(new_rel);
@@ -849,6 +899,22 @@ merge_relinfos(RelationMergeInfo *relinfos, int nrelids, int mergeindex)
 	table_close(new_rel, NoLock);
 	table_close(relRelation, RowExclusiveLock);
 
+	/*
+	 * Update compression chunk size stats, but only if at least one of the
+	 * merged chunks was compressed. In that case the merged metadata should
+	 * be non-zero.
+	 */
+	if (merged_ccs.compressed_heap_size > 0)
+	{
+		/*
+		 * The result relation should always be compressed because we pick the
+		 * first compressed one, if one exists.
+		 */
+
+		Assert(result_minfo->ccs.compressed_heap_size > 0);
+		ts_compression_chunk_size_update(result_minfo->chunk->fd.id, &merged_ccs);
+	}
+
 	return new_relid;
 }
 
@@ -1026,6 +1092,22 @@ chunk_merge_chunks(PG_FUNCTION_ARGS)
 					(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 					 errmsg("can only merge hypertable chunks")));
 
+		if (!merge_chunks_multidim_allowed())
+		{
+			Cache *hcache;
+			Hypertable *ht = ts_hypertable_cache_get_cache_and_entry(chunk->hypertable_relid,
+																	 CACHE_FLAG_NONE,
+																	 &hcache);
+			Ensure(ht, "missing hypertable for chunk");
+
+			if (ht->fd.num_dimensions > 1)
+				ereport(ERROR,
+						(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+						 errmsg("cannot merge chunk in multi-dimensional hypertable")));
+
+			ts_cache_release(&hcache);
+		}
+
 		if (chunk->fd.osm_chunk)
 			ereport(ERROR,
 					(errcode(ERRCODE_FEATURE_NOT_SUPPORTED), errmsg("cannot merge OSM chunks")));
@@ -1044,6 +1126,14 @@ chunk_merge_chunks(PG_FUNCTION_ARGS)
 
 			if (mergeindex == -1)
 				mergeindex = i;
+
+			/* Read compression chunk size stats */
+			bool found = ts_compression_chunk_size_get(chunk->fd.id, &relinfo->ccs);
+
+			if (!found)
+				elog(WARNING,
+					 "missing compression chunk size stats for compressed chunk \"%s\"",
+					 NameStr(chunk->fd.table_name));
 		}
 
 		if (ts_chunk_is_frozen(chunk))
@@ -1180,3 +1270,556 @@ chunk_merge_chunks(PG_FUNCTION_ARGS)
 
 	PG_RETURN_VOID();
 }
+
+typedef struct RelationSplitInfo
+{
+	BulkInsertState bistate;
+	TupleTableSlot *dstslot;
+	RewriteState rwstate;
+	Relation targetrel;
+	Datum *values;
+	bool *isnull;
+	/*
+	 * Tuple mapping is needed in case the old relation has dropped
+	 * columns. New relations (as result of split) are "clean" without dropped
+	 * columns. The tuple map converts tuples between the source and
+	 * destination chunks.
+	 */
+	TupleConversionMap *tupmap;
+} RelationSplitInfo;
+
+static RelationSplitInfo *
+relation_split_info_create(Relation srcrel, Relation targetrel, struct VacuumCutoffs *cutoffs)
+{
+	RelationSplitInfo *rsi = palloc0(sizeof(RelationSplitInfo));
+
+	rsi->targetrel = targetrel;
+	rsi->bistate = GetBulkInsertState();
+	rsi->rwstate = begin_heap_rewrite(srcrel,
+									  targetrel,
+									  cutoffs->OldestXmin,
+									  cutoffs->FreezeLimit,
+									  cutoffs->MultiXactCutoff);
+
+	rsi->tupmap = convert_tuples_by_name(RelationGetDescr(srcrel), RelationGetDescr(targetrel));
+
+	/* Create tuple slot for new partition. */
+	rsi->dstslot =
+		MakeSingleTupleTableSlot(RelationGetDescr(targetrel), table_slot_callbacks(targetrel));
+	ExecStoreAllNullTuple(rsi->dstslot);
+
+	rsi->values = (Datum *) palloc0(RelationGetDescr(srcrel)->natts * sizeof(Datum));
+	rsi->isnull = (bool *) palloc0(RelationGetDescr(srcrel)->natts * sizeof(bool));
+
+	return rsi;
+}
+
+static void
+relation_split_info_free(RelationSplitInfo *rsi, int ti_options)
+{
+	ExecDropSingleTupleTableSlot(rsi->dstslot);
+	FreeBulkInsertState(rsi->bistate);
+	table_finish_bulk_insert(rsi->targetrel, ti_options);
+	end_heap_rewrite(rsi->rwstate);
+	table_close(rsi->targetrel, NoLock);
+	pfree(rsi->values);
+	pfree(rsi->isnull);
+
+	if (rsi->tupmap)
+		free_conversion_map(rsi->tupmap);
+
+	rsi->targetrel = NULL;
+	rsi->bistate = NULL;
+	rsi->dstslot = NULL;
+	rsi->tupmap = NULL;
+	rsi->values = NULL;
+	rsi->isnull = NULL;
+	pfree(rsi);
+}
+
+/*
+ * Reconstruct and rewrite the given tuple.
+ *
+ * Mostly taken from heapam module.
+ *
+ * When splitting a relation in two, the old relation is retained for one of
+ * the result relations while the other is created new. This might lead to a
+ * situation where the two result relations have different attribute mappings
+ * because the old one could have dropped columns while the new one is "clean"
+ * without dropped columns. Therefore, the rewrite function needs to account
+ * for this when the tuple is rewritten.
+ */
+static void
+reform_and_rewrite_tuple(HeapTuple tuple, Relation srcrel, RelationSplitInfo *rsi)
+{
+	TupleDesc oldTupDesc = RelationGetDescr(srcrel);
+	TupleDesc newTupDesc = RelationGetDescr(rsi->targetrel);
+	HeapTuple tupcopy;
+
+	if (rsi->tupmap)
+	{
+		/*
+		 * If this is the "new" relation, the tuple map might be different
+		 * from the "source" relation.
+		 */
+		tupcopy = execute_attr_map_tuple(tuple, rsi->tupmap);
+	}
+	else
+	{
+		int i;
+
+		heap_deform_tuple(tuple, oldTupDesc, rsi->values, rsi->isnull);
+
+		/* Be sure to null out any dropped columns if this is the "old"
+		 * relation. A relation created new doesn't have dropped columns. */
+		for (i = 0; i < newTupDesc->natts; i++)
+		{
+			if (TupleDescAttr(newTupDesc, i)->attisdropped)
+				rsi->isnull[i] = true;
+		}
+
+		tupcopy = heap_form_tuple(newTupDesc, rsi->values, rsi->isnull);
+	}
+
+	/* The heap rewrite module does the rest */
+	rewrite_heap_tuple(rsi->rwstate, tuple, tupcopy);
+
+	heap_freetuple(tupcopy);
+}
+
+/*
+ * Split a chunk along a given dimension and split point.
+ *
+ * The column/dimension and "split at" point are optional. If these arguments
+ * are not specified, the chunk is split in two equal ranges based on the
+ * primary partitioning column.
+ *
+ * The split is done using the table rewrite approach used by the PostgreSQL
+ * CLUSTER code (also used for VACUUM FULL). It uses the rewrite module to
+ * retain the visibility information of tuples, and also transferring (old)
+ * deleted or updated tuples that are still visible to concurrent transactions
+ * reading an older snapshot. Completely dead tuples are garbage collected.
+ *
+ * The advantage of the rewrite approach is that it is fully MVCC compliant
+ * and ensures the result relations have minimal garbage after the split. Note
+ * that locks don't fully protect against visibility issues since a concurrent
+ * transaction can be pinned to an older snapshot while not (yet) holding any
+ * locks on relations (chunks and hypertables) being split.
+ */
+Datum
+chunk_split_chunk(PG_FUNCTION_ARGS)
+{
+	Oid relid = PG_ARGISNULL(0) ? InvalidOid : PG_GETARG_OID(0);
+	Relation srcrel;
+
+	srcrel = table_open(relid, AccessExclusiveLock);
+
+	if (srcrel->rd_rel->relkind != RELKIND_RELATION)
+		ereport(ERROR,
+				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+				 errmsg("cannot split non-table relations")));
+
+	Oid amoid = srcrel->rd_rel->relam;
+
+	if (amoid != HEAP_TABLE_AM_OID)
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+				 errmsg("access method \"%s\" is not supported for split", get_am_name(amoid))));
+
+	/* Only owner is allowed to split */
+	if (!object_ownercheck(RelationRelationId, relid, GetUserId()))
+		aclcheck_error(ACLCHECK_NOT_OWNER,
+					   get_relkind_objtype(srcrel->rd_rel->relkind),
+					   get_rel_name(relid));
+
+	/* Lock toast table to prevent it from being concurrently vacuumed */
+	if (srcrel->rd_rel->reltoastrelid)
+		LockRelationOid(srcrel->rd_rel->reltoastrelid, AccessExclusiveLock);
+
+	/*
+	 * Check for active uses of the relation in the current transaction,
+	 * including open scans and pending AFTER trigger events.
+	 */
+	CheckTableNotInUse(srcrel, "split_chunk");
+
+	const Chunk *chunk = ts_chunk_get_by_relid(relid, true);
+
+	if (chunk->fd.compressed_chunk_id != INVALID_CHUNK_ID)
+		ereport(ERROR,
+				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+				 errmsg("splitting a compressed chunk is not supported"),
+				 errhint("Decompress the chunk before splitting it.")));
+
+	if (chunk->fd.osm_chunk)
+		ereport(ERROR, (errcode(ERRCODE_FEATURE_NOT_SUPPORTED), errmsg("cannot split OSM chunks")));
+
+	if (ts_chunk_is_frozen(chunk))
+		ereport(ERROR,
+				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+				 errmsg("cannot split frozen chunk \"%s.%s\" scheduled for tiering",
+						NameStr(chunk->fd.schema_name),
+						NameStr(chunk->fd.table_name)),
+				 errhint("Untier the chunk before splitting it.")));
+
+	Cache *hcache;
+	const Hypertable *ht =
+		ts_hypertable_cache_get_cache_and_entry(chunk->hypertable_relid, CACHE_FLAG_NONE, &hcache);
+	const Dimension *dim = hyperspace_get_open_dimension(ht->space, 0);
+
+	Ensure(dim, "no primary dimension for chunk");
+
+	if (ht->fd.num_dimensions > 1)
+		ereport(ERROR,
+				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+				 errmsg("cannot split chunk in multi-dimensional hypertable")));
+
+	AttrNumber splitdim_attnum = get_attnum(relid, NameStr(dim->fd.column_name));
+	Oid splitdim_type = get_atttype(relid, splitdim_attnum);
+	Datum split_at;
+	bool have_split_at = false;
+
+	/* Check split_at argument */
+	if (!PG_ARGISNULL(1))
+	{
+		Oid argtype = get_fn_expr_argtype(fcinfo->flinfo, 1);
+		Datum arg = PG_GETARG_DATUM(1);
+
+		if (argtype == UNKNOWNOID)
+		{
+			Oid infuncid = InvalidOid;
+			Oid typioparam;
+
+			getTypeInputInfo(splitdim_type, &infuncid, &typioparam);
+
+			switch (get_func_nargs(infuncid))
+			{
+				case 1:
+					/* Functions that take one input argument, e.g., the Date function */
+					split_at = OidFunctionCall1(infuncid, arg);
+					break;
+				case 3:
+					/* Timestamp functions take three input arguments */
+					split_at = OidFunctionCall3(infuncid,
+												arg,
+												ObjectIdGetDatum(InvalidOid),
+												Int32GetDatum(-1));
+					break;
+				default:
+					/* Shouldn't be any time types with other number of args */
+					Ensure(false, "invalid type for split_at");
+					pg_unreachable();
+			}
+			argtype = splitdim_type;
+		}
+		else
+			split_at = arg;
+
+		if (argtype != splitdim_type)
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+					 errmsg("invalid type '%s' for split_at argument", format_type_be(argtype)),
+					 errdetail("The argument type must match the dimension \"%s\"",
+							   NameStr(dim->fd.column_name))));
+
+		have_split_at = true;
+	}
+
+	/* Serialize chunk creation around the root hypertable. NOTE: also taken
+	 * in ts_chunk_find_or_create_without_cuts() below. */
+	LockRelationOid(ht->main_table_relid, ShareUpdateExclusiveLock);
+
+	/*
+	 * Find the existing partition slice for the chunk being split.
+	 */
+	DimensionSlice *slice = NULL;
+	Hypercube *new_cube = ts_hypercube_copy(chunk->cube);
+
+	for (int i = 0; i < new_cube->num_slices; i++)
+	{
+		DimensionSlice *curr_slice = new_cube->slices[i];
+
+		if (curr_slice->fd.dimension_id == dim->fd.id)
+		{
+			slice = curr_slice;
+			break;
+		}
+	}
+
+	Ensure(slice, "no chunk slice for dimension %s", NameStr(dim->fd.column_name));
+
+	/*
+	 * Pick split point and calculate new ranges. If no split point is given
+	 * by the user, then split in the middle.
+	 */
+	int64 interval_range = slice->fd.range_end - slice->fd.range_start;
+	int64 split_point = 0;
+
+	if (have_split_at)
+	{
+		split_point = ts_time_value_to_internal(split_at, splitdim_type);
+
+		/*
+		 * Check that the split_at value actually produces a valid split. Note
+		 * that range_start is inclusive while range_end is non-inclusive. The
+		 * split_at value needs to produce partition ranges of at least length
+		 * 1.
+		 */
+		if (split_point < (slice->fd.range_start + 1) || split_point > (slice->fd.range_end - 2))
+		{
+			Oid outfuncid = InvalidOid;
+			bool isvarlena = false;
+
+			getTypeOutputInfo(splitdim_type, &outfuncid, &isvarlena);
+			Datum split_at_str = OidFunctionCall1(outfuncid, split_at);
+
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+					 errmsg("cannot split chunk at %s", DatumGetCString(split_at_str))));
+		}
+	}
+	else
+		split_point = slice->fd.range_start + (interval_range / 2);
+
+	int64 old_end = slice->fd.range_end;
+
+	/* Update the slice range for the existing chunk */
+	slice->fd.range_end = split_point;
+	chunk_update_constraints(chunk, new_cube);
+
+	/* Update the slice for the new chunk */
+	slice->fd.range_start = split_point;
+	slice->fd.range_end = old_end;
+	slice->fd.id = 0; /* Must set to 0 to mark as new for it to be created */
+
+	/* Make updated constraints visible */
+	CommandCounterIncrement();
+
+	/* Reread hypertable after constraints changed */
+	ts_cache_release(&hcache);
+	ht = ts_hypertable_cache_get_cache_and_entry(chunk->hypertable_relid, CACHE_FLAG_NONE, &hcache);
+	bool created = false;
+	Chunk *new_chunk = ts_chunk_find_or_create_without_cuts(ht,
+															new_cube,
+															NameStr(chunk->fd.schema_name),
+															NULL,
+															InvalidOid,
+															&created);
+	ts_cache_release(&hcache);
+
+	Ensure(created, "could not create chunk for split");
+	Assert(new_chunk);
+
+	int ti_options = TABLE_INSERT_SKIP_FSM;
+	TupleTableSlot *srcslot;
+	TableScanDesc scan;
+	MemoryContext oldcxt;
+	EState *estate;
+	ExprContext *econtext;
+	RelationSplitInfo *splitinfos[2];
+	struct VacuumCutoffs cutoffs;
+	char relpersistence = srcrel->rd_rel->relpersistence;
+
+	compute_rel_vacuum_cutoffs(srcrel, &cutoffs);
+
+	Oid new_heap_relid = make_new_heap(RelationGetRelid(srcrel),
+									   srcrel->rd_rel->reltablespace,
+									   srcrel->rd_rel->relam,
+									   relpersistence,
+									   AccessExclusiveLock);
+
+	estate = CreateExecutorState();
+
+	/* Create the tuple slot */
+	srcslot = MakeSingleTupleTableSlot(RelationGetDescr(srcrel), table_slot_callbacks(srcrel));
+
+	/*
+	 * Open the new relations that will receive tuples during split. These are
+	 * closed by relation_split_info_free().
+	 */
+	Relation target1_rel = table_open(new_heap_relid, AccessExclusiveLock);
+	Relation target2_rel = table_open(new_chunk->table_id, AccessExclusiveLock);
+
+	/* Setup the state we need for each new chunk partition */
+	splitinfos[0] = relation_split_info_create(srcrel, target1_rel, &cutoffs);
+	splitinfos[1] = relation_split_info_create(srcrel, target2_rel, &cutoffs);
+
+	DEBUG_WAITPOINT("split_chunk_before_tuple_routing");
+
+	/*
+	 * Scan through the rows using SnapshotAny to see everything so that we
+	 * can transfer tuples that are deleted or updated but still visible to
+	 * concurrent transactions.
+	 */
+	scan = table_beginscan(srcrel, SnapshotAny, 0, NULL);
+
+	/*
+	 * Switch to per-tuple memory context and reset it for each tuple
+	 * produced, so we don't leak memory.
+	 */
+	econtext = GetPerTupleExprContext(estate);
+	oldcxt = MemoryContextSwitchTo(GetPerTupleMemoryContext(estate));
+
+	/*
+	 * Read all the data from the split relation and route the tuples to the
+	 * new partitions. Do some vacuuming and cleanup at the same
+	 * time. Transfer all visibility information to the new relations.
+	 *
+	 * Main loop inspired by heapam_relation_copy_for_cluster() used to run
+	 * CLUSTER and VACUUM FULL on a table.
+	 */
+	double num_tuples = 0.0;
+	double tups_vacuumed = 0.0;
+	double tups_recently_dead = 0.0;
+
+	BufferHeapTupleTableSlot *hslot = (BufferHeapTupleTableSlot *) srcslot;
+
+	while (table_scan_getnextslot(scan, ForwardScanDirection, srcslot))
+	{
+		RelationSplitInfo *rsi = NULL;
+		bool isnull = true;
+		HeapTuple tuple;
+		Buffer buf;
+		bool isdead;
+
+		CHECK_FOR_INTERRUPTS();
+		ResetExprContext(econtext);
+
+		tuple = ExecFetchSlotHeapTuple(srcslot, false, NULL);
+		buf = hslot->buffer;
+
+		LockBuffer(buf, BUFFER_LOCK_SHARE);
+
+		switch (HeapTupleSatisfiesVacuum(tuple, cutoffs.OldestXmin, buf))
+		{
+			case HEAPTUPLE_DEAD:
+				/* Definitely dead */
+				isdead = true;
+				break;
+			case HEAPTUPLE_RECENTLY_DEAD:
+				tups_recently_dead += 1;
+				/* fall through */
+				TS_FALLTHROUGH;
+			case HEAPTUPLE_LIVE:
+				/* Live or recently dead, must copy it */
+				isdead = false;
+				break;
+			case HEAPTUPLE_INSERT_IN_PROGRESS:
+				/*
+				 * Since we hold exclusive lock on the relation, normally the
+				 * only way to see this is if it was inserted earlier in our
+				 * own transaction. Give a warning if this case does not
+				 * apply; in any case we better copy it.
+				 */
+				if (!TransactionIdIsCurrentTransactionId(HeapTupleHeaderGetXmin(tuple->t_data)))
+					elog(WARNING,
+						 "concurrent insert in progress within table \"%s\"",
+						 RelationGetRelationName(srcrel));
+				/* treat as live */
+				isdead = false;
+				break;
+			case HEAPTUPLE_DELETE_IN_PROGRESS:
+				/*
+				 * Similar situation to INSERT_IN_PROGRESS case.
+				 */
+				if (!TransactionIdIsCurrentTransactionId(
+						HeapTupleHeaderGetUpdateXid(tuple->t_data)))
+					elog(WARNING,
+						 "concurrent delete in progress within table \"%s\"",
+						 RelationGetRelationName(srcrel));
+				/* treat as recently dead */
+				tups_recently_dead += 1;
+				isdead = false;
+				break;
+			default:
+				elog(ERROR, "unexpected HeapTupleSatisfiesVacuum result");
+				isdead = false; /* keep compiler quiet */
+				break;
+		}
+
+		LockBuffer(buf, BUFFER_LOCK_UNLOCK);
+
+		/* Find the partition/chunk to insert the tuple into */
+		Datum value = slot_getattr(srcslot, splitdim_attnum, &isnull);
+		Assert(!isnull);
+		int64 point = ts_time_value_to_internal(value, splitdim_type);
+
+		/*
+		 * Route to partition based on new boundaries. Only 2-way split is
+		 * supported now, so routing is easy. An N-way split requires, e.g.,
+		 * binary search.
+		 */
+		if (point < split_point)
+			rsi = splitinfos[0];
+		else
+			rsi = splitinfos[1];
+
+		if (isdead)
+		{
+			tups_vacuumed += 1;
+			/* heap rewrite module still needs to see it... */
+			if (rewrite_heap_dead_tuple(rsi->rwstate, tuple))
+			{
+				/* A previous recently-dead tuple is now known dead */
+				tups_vacuumed += 1;
+				tups_recently_dead -= 1;
+			}
+			continue;
+		}
+
+		num_tuples++;
+		reform_and_rewrite_tuple(tuple, srcrel, rsi);
+	}
+
+	MemoryContextSwitchTo(oldcxt);
+
+	const char *nspname = get_namespace_name(RelationGetNamespace(srcrel));
+
+	ereport(DEBUG1,
+			(errmsg("\"%s.%s\": found %.0f removable, %.0f nonremovable row versions",
+					nspname,
+					RelationGetRelationName(srcrel),
+					tups_vacuumed,
+					num_tuples),
+			 errdetail("%.0f dead row versions cannot be removed yet.", tups_recently_dead)));
+
+	table_endscan(scan);
+	ExecDropSingleTupleTableSlot(srcslot);
+	FreeExecutorState(estate);
+
+	table_close(srcrel, NoLock);
+
+	/* Cleanup split rel infos and reindex new chunks */
+	for (int i = 0; i < 2; i++)
+	{
+		ReindexParams reindex_params = { 0 };
+		int reindex_flags = REINDEX_REL_SUPPRESS_INDEX_USE;
+		Oid chunkrelid = splitinfos[i]->targetrel->rd_id;
+
+		Ensure(relpersistence == RELPERSISTENCE_PERMANENT, "only permanent chunks can be split");
+		reindex_flags |= REINDEX_REL_FORCE_INDEXES_PERMANENT;
+
+		relation_split_info_free(splitinfos[i], ti_options);
+
+		/* Only reindex new chunks. Existing chunk will be reindexed during
+		 * the heap swap below. */
+		if (i > 0)
+			reindex_relation_compat(NULL, chunkrelid, reindex_flags, &reindex_params);
+	}
+
+	/* Finally, swap the heap of the chunk that we split so that it only
+	 * contains the tuples for its new partition boundaries. AccessExclusive
+	 * lock is held during the swap. */
+	finish_heap_swap(relid,
+					 new_heap_relid,
+					 false, /* system catalog */
+					 false /* swap toast by content */,
+					 true, /* check constraints */
+					 true, /* internal? */
+					 cutoffs.FreezeLimit,
+					 cutoffs.MultiXactCutoff,
+					 relpersistence);
+
+	DEBUG_WAITPOINT("split_chunk_at_end");
+
+	PG_RETURN_VOID();
+}
