@@ -245,15 +245,17 @@ compresschunkcxt_init(CompressChunkCxt *cxt, Cache *hcache, Oid hypertable_relid
 		get_hypertable_or_cagg_name(srcht, &cagg_ht_name);
 		ereport(ERROR,
 				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-				 errmsg("compression not enabled on \"%s\"", NameStr(cagg_ht_name)),
-				 errdetail("It is not possible to compress chunks on a hypertable or"
-						   " continuous aggregate that does not have compression enabled."),
-				 errhint("Enable compression using ALTER TABLE/MATERIALIZED VIEW with"
-						 " the timescaledb.compress option.")));
+				 errmsg("columnstore not enabled on \"%s\"", NameStr(cagg_ht_name)),
+				 errdetail("It is not possible to convert chunks to columnstore on a hypertable or"
+						   " continuous aggregate that does not have columnstore enabled."),
+				 errhint("Enable columnstore using ALTER TABLE/MATERIALIZED VIEW with"
+						 " the timescaledb.enable_columnstore option.")));
 	}
 	compress_ht = ts_hypertable_get_by_id(srcht->fd.compressed_hypertable_id);
 	if (compress_ht == NULL)
-		ereport(ERROR, (errcode(ERRCODE_INTERNAL_ERROR), errmsg("missing compress hypertable")));
+		ereport(ERROR,
+				(errcode(ERRCODE_INTERNAL_ERROR),
+				 errmsg("missing columnstore-enabled hypertable")));
 	/* user has to be the owner of the compression table too */
 	ts_hypertable_permissions_check(compress_ht->main_table_relid, GetUserId());
 
@@ -350,6 +352,9 @@ find_chunk_to_merge_into(Hypertable *ht, Chunk *current_chunk)
  * after the existing data according to the compression order. This is true if the data being merged
  * in has timestamps greater than the existing data and the first column in the order by is time
  * ASC.
+ *
+ * The CompressChunkCxt references the chunk we are merging and mergable_chunk is the chunk we
+ * are merging into.
  */
 static bool
 check_is_chunk_order_violated_by_merge(CompressChunkCxt *cxt, const Dimension *time_dim,
@@ -362,14 +367,13 @@ check_is_chunk_order_violated_by_merge(CompressChunkCxt *cxt, const Dimension *t
 	const DimensionSlice *compressed_slice =
 		ts_hypercube_get_slice_by_dimension_id(cxt->srcht_chunk->cube, time_dim->fd.id);
 	if (!compressed_slice)
-		elog(ERROR, "compressed chunk has no time dimension slice");
-
-	if (mergable_slice->fd.range_start > compressed_slice->fd.range_start &&
-		mergable_slice->fd.range_end > compressed_slice->fd.range_start)
-	{
-		return true;
-	}
-
+		elog(ERROR, "columnstore chunk has no time dimension slice");
+	/*
+	 * Ensure the compressed chunk is AFTER the chunk that
+	 * it is being merged into. This is already guaranteed by previous checks.
+	 */
+	Ensure(mergable_slice->fd.range_end == compressed_slice->fd.range_start,
+		   "chunk being merged is not after the chunk that is being merged into");
 	CompressionSettings *ht_settings =
 		ts_compression_settings_get(mergable_chunk->hypertable_relid);
 
@@ -399,7 +403,7 @@ compress_chunk_impl(Oid hypertable_relid, Oid chunk_relid)
 
 	/* acquire locks on src and compress hypertable and src chunk */
 	ereport(DEBUG1,
-			(errmsg("acquiring locks for compressing \"%s.%s\"",
+			(errmsg("acquiring locks for converting to columnstore \"%s.%s\"",
 					get_namespace_name(get_rel_namespace(chunk_relid)),
 					get_rel_name(chunk_relid))));
 	LockRelationOid(cxt.srcht->main_table_relid, AccessShareLock);
@@ -409,7 +413,7 @@ compress_chunk_impl(Oid hypertable_relid, Oid chunk_relid)
 	/* acquire locks on catalog tables to keep till end of txn */
 	LockRelationOid(catalog_get_table_id(ts_catalog_get(), CHUNK), RowExclusiveLock);
 	ereport(DEBUG1,
-			(errmsg("locks acquired for compressing \"%s.%s\"",
+			(errmsg("locks acquired for converting to columnstore \"%s.%s\"",
 					get_namespace_name(get_rel_namespace(chunk_relid)),
 					get_rel_name(chunk_relid))));
 
@@ -446,7 +450,7 @@ compress_chunk_impl(Oid hypertable_relid, Oid chunk_relid)
 		ts_chunk_set_compressed_chunk(cxt.srcht_chunk, compress_ht_chunk->fd.id);
 		new_compressed_chunk = true;
 		ereport(DEBUG1,
-				(errmsg("new compressed chunk \"%s.%s\" created",
+				(errmsg("new columnstore chunk \"%s.%s\" created",
 						NameStr(compress_ht_chunk->fd.schema_name),
 						NameStr(compress_ht_chunk->fd.table_name))));
 
@@ -471,7 +475,7 @@ compress_chunk_impl(Oid hypertable_relid, Oid chunk_relid)
 		compress_ht_chunk = ts_chunk_get_by_id(mergable_chunk->fd.compressed_chunk_id, true);
 		result_chunk_id = mergable_chunk->table_id;
 		ereport(DEBUG1,
-				(errmsg("merge into existing compressed chunk \"%s.%s\"",
+				(errmsg("merge into existing columnstore chunk \"%s.%s\"",
 						NameStr(compress_ht_chunk->fd.schema_name),
 						NameStr(compress_ht_chunk->fd.table_name))));
 	}
@@ -536,6 +540,25 @@ compress_chunk_impl(Oid hypertable_relid, Oid chunk_relid)
 		 */
 		ts_chunk_constraints_create(cxt.compress_ht, compress_ht_chunk);
 		ts_trigger_create_all_on_chunk(compress_ht_chunk);
+
+		/* Detect and emit warning if poor compression ratio is found */
+		float compression_ratio = ((float) before_size.total_size / after_size.total_size);
+		float POOR_COMPRESSION_THRESHOLD = 1.0;
+		if (ts_guc_enable_compression_ratio_warnings &&
+			compression_ratio < POOR_COMPRESSION_THRESHOLD)
+			ereport(WARNING,
+					errcode(ERRCODE_WARNING),
+					errmsg("poor compression rate detected for chunk \"%s\"'",
+						   get_rel_name(chunk_relid)),
+					errdetail("Chunk \"%s\" has a poor compression ratio: %.2f. Size before "
+							  "compression: " INT64_FORMAT
+							  " bytes. Size after compression: " INT64_FORMAT " bytes",
+							  get_rel_name(chunk_relid),
+							  compression_ratio,
+							  before_size.total_size,
+							  after_size.total_size),
+					errhint("Changing compression settings for \"%s\" can improve compression rate",
+							get_rel_name(hypertable_relid)));
 	}
 	else
 	{
@@ -561,7 +584,7 @@ compress_chunk_impl(Oid hypertable_relid, Oid chunk_relid)
 		}
 	}
 
-	ts_cache_release(hcache);
+	ts_cache_release(&hcache);
 	return result_chunk_id;
 }
 
@@ -581,22 +604,25 @@ decompress_chunk_impl(Chunk *uncompressed_chunk, bool if_compressed)
 	if (TS_HYPERTABLE_IS_INTERNAL_COMPRESSION_TABLE(uncompressed_hypertable))
 		ereport(ERROR,
 				(errcode(ERRCODE_INTERNAL_ERROR),
-				 errmsg("decompress_chunk must not be called on the internal compressed chunk")));
+				 errmsg(
+					 "convert_to_rowstore must not be called on the internal columnstore chunk")));
 
 	compressed_hypertable =
 		ts_hypertable_get_by_id(uncompressed_hypertable->fd.compressed_hypertable_id);
 	if (compressed_hypertable == NULL)
-		ereport(ERROR, (errcode(ERRCODE_INTERNAL_ERROR), errmsg("missing compressed hypertable")));
+		ereport(ERROR,
+				(errcode(ERRCODE_INTERNAL_ERROR),
+				 errmsg("missing columnstore-enabled hypertable")));
 
 	if (uncompressed_chunk->fd.hypertable_id != uncompressed_hypertable->fd.id)
 		elog(ERROR, "hypertable and chunk do not match");
 
 	if (uncompressed_chunk->fd.compressed_chunk_id == INVALID_CHUNK_ID)
 	{
-		ts_cache_release(hcache);
+		ts_cache_release(&hcache);
 		ereport((if_compressed ? NOTICE : ERROR),
 				(errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
-				 errmsg("chunk \"%s\" is not compressed",
+				 errmsg("chunk \"%s\" is not converted to columnstore",
 						get_rel_name(uncompressed_chunk->table_id))));
 		return;
 	}
@@ -607,7 +633,7 @@ decompress_chunk_impl(Chunk *uncompressed_chunk, bool if_compressed)
 	compressed_chunk = ts_chunk_get_by_id(uncompressed_chunk->fd.compressed_chunk_id, true);
 
 	ereport(DEBUG1,
-			(errmsg("acquiring locks for decompressing \"%s.%s\"",
+			(errmsg("acquiring locks for converting to rowstore \"%s.%s\"",
 					NameStr(uncompressed_chunk->fd.schema_name),
 					NameStr(uncompressed_chunk->fd.table_name))));
 	/* acquire locks on src and compress hypertable and src chunk */
@@ -634,7 +660,7 @@ decompress_chunk_impl(Chunk *uncompressed_chunk, bool if_compressed)
 	/* acquire locks on catalog tables to keep till end of txn */
 	LockRelationOid(catalog_get_table_id(ts_catalog_get(), CHUNK), RowExclusiveLock);
 	ereport(DEBUG1,
-			(errmsg("locks acquired for decompressing \"%s.%s\"",
+			(errmsg("locks acquired for converting to rowstore \"%s.%s\"",
 					NameStr(uncompressed_chunk->fd.schema_name),
 					NameStr(uncompressed_chunk->fd.table_name))));
 
@@ -671,7 +697,7 @@ decompress_chunk_impl(Chunk *uncompressed_chunk, bool if_compressed)
 	LockRelationOid(uncompressed_chunk->table_id, AccessExclusiveLock);
 	LockRelationOid(compressed_chunk->table_id, AccessExclusiveLock);
 	ts_chunk_drop(compressed_chunk, DROP_RESTRICT, -1);
-	ts_cache_release(hcache);
+	ts_cache_release(&hcache);
 	write_logical_replication_msg_decompression_end();
 }
 
@@ -751,7 +777,7 @@ tsl_create_compressed_chunk(PG_FUNCTION_ARGS)
 		 */
 		ts_chunk_set_partial(cxt.srcht_chunk);
 	}
-	ts_cache_release(hcache);
+	ts_cache_release(&hcache);
 
 	PG_RETURN_OID(chunk_relid);
 }
@@ -759,7 +785,6 @@ tsl_create_compressed_chunk(PG_FUNCTION_ARGS)
 static Oid
 set_access_method(Oid relid, const char *amname)
 {
-#if PG15_GE
 	AlterTableCmd cmd = {
 		.type = T_AlterTableCmd,
 		.subtype = AT_SetAccessMethod,
@@ -804,11 +829,6 @@ set_access_method(Oid relid, const char *amname)
 #endif
 	hypercore_alter_access_method_finish(relid, !to_hypercore);
 
-#else
-	ereport(ERROR,
-			(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-			 errmsg("compression using hypercore is not supported")));
-#endif
 	return relid;
 }
 
@@ -853,10 +873,11 @@ compress_hypercore(Chunk *chunk, bool rel_is_hypercore, UseAccessMethod useam,
 			Assert(rel_is_hypercore);
 			ereport(ERROR,
 					(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-					 errmsg("cannot compress \"%s\" without using Hypercore access method",
+					 errmsg("cannot converting to columnstore \"%s\" without using Hypercore "
+							"access method",
 							get_rel_name(chunk->table_id)),
-					 errhint(
-						 "Decompress first and then compress without Hypercore access method.")));
+					 errhint("Convert to rowstore first and then convert to columnstore without "
+							 "Hypercore access method.")));
 			break;
 		case USE_AM_NULL:
 			Assert(rel_is_hypercore);
@@ -952,7 +973,8 @@ tsl_compress_chunk_wrapper(Chunk *chunk, bool if_not_compressed, bool recompress
 			write_logical_replication_msg_compression_end();
 			ereport((if_not_compressed ? NOTICE : ERROR),
 					(errcode(ERRCODE_DUPLICATE_OBJECT),
-					 errmsg("chunk \"%s\" is already compressed", get_rel_name(chunk->table_id))));
+					 errmsg("chunk \"%s\" is already converted to columnstore",
+							get_rel_name(chunk->table_id))));
 			return uncompressed_chunk_id;
 		}
 
@@ -1005,7 +1027,9 @@ tsl_decompress_chunk(PG_FUNCTION_ARGS)
 	ts_hypertable_permissions_check(ht->main_table_relid, GetUserId());
 
 	if (!ht->fd.compressed_hypertable_id)
-		ereport(ERROR, (errcode(ERRCODE_INTERNAL_ERROR), errmsg("missing compressed hypertable")));
+		ereport(ERROR,
+				(errcode(ERRCODE_INTERNAL_ERROR),
+				 errmsg("missing columnstore-enabled hypertable")));
 
 	if (ts_is_hypercore_am(uncompressed_chunk->amoid))
 		set_access_method(uncompressed_chunk_id, "heap");
@@ -1013,7 +1037,8 @@ tsl_decompress_chunk(PG_FUNCTION_ARGS)
 	{
 		ereport((if_compressed ? NOTICE : ERROR),
 				(errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
-				 errmsg("chunk \"%s\" is not compressed", get_rel_name(uncompressed_chunk_id))));
+				 errmsg("chunk \"%s\" is not converted to columnstore",
+						get_rel_name(uncompressed_chunk_id))));
 
 		PG_RETURN_NULL();
 	}
