@@ -12,6 +12,7 @@
 #include <libpq/pqformat.h>
 #include <storage/predicate.h>
 #include <utils/datum.h>
+#include <utils/elog.h>
 #include <utils/snapmgr.h>
 #include <utils/syscache.h>
 #include <utils/typcache.h>
@@ -38,6 +39,15 @@
 #include "ts_catalog/catalog.h"
 #include "ts_catalog/compression_chunk_size.h"
 #include "ts_catalog/compression_settings.h"
+#include <nodes/decompress_chunk/vector_quals.h>
+
+/*
+ * Timing parameters for truncate locking heuristics.
+ * These are the same as used by Postgres for truncate locking during lazy vacuum.
+ * https://github.com/postgres/postgres/blob/4a0650d359c5981270039eeb634c3b7427aa0af5/src/backend/access/heap/vacuumlazy.c#L82
+ */
+#define COMPRESS_TRUNCATE_LOCK_WAIT_INTERVAL 50 /* ms */
+#define COMPRESS_TRUNCATE_LOCK_TIMEOUT 5000		/* ms */
 
 StaticAssertDecl(GLOBAL_MAX_ROWS_PER_COMPRESSION >= TARGET_COMPRESSED_BATCH_SIZE,
 				 "max row numbers must be harmonized");
@@ -97,7 +107,7 @@ tsl_get_decompress_all_function(CompressionAlgorithm algorithm, Oid type)
 	if (algorithm >= _END_COMPRESSION_ALGORITHMS)
 		elog(ERROR, "invalid compression algorithm %d", algorithm);
 
-	if (type != TEXTOID &&
+	if (type != TEXTOID && type != BOOLOID &&
 		(algorithm == COMPRESSION_ALGORITHM_DICTIONARY || algorithm == COMPRESSION_ALGORITHM_ARRAY))
 	{
 		/* Bulk decompression of array and dictionary is only supported for text. */
@@ -263,8 +273,6 @@ compress_chunk(Oid in_table, Oid out_table, int insert_options)
 	HeapTuple in_table_tp = NULL, index_tp = NULL;
 	Form_pg_attribute in_table_attr_tp, index_attr_tp;
 	CompressionStats cstat;
-	/* Might be merging into an existing chunk, so get compression settings
-	 * from that chunk */
 	CompressionSettings *settings = ts_compression_settings_get_by_compress_relid(out_table);
 
 	int64 report_reltuples;
@@ -464,7 +472,7 @@ compress_chunk(Oid in_table, Oid out_table, int insert_options)
 
 		Assert(!REL_IS_HYPERCORE(in_rel));
 		elog(ts_guc_debug_compression_path_info ? INFO : DEBUG1,
-			 "using index \"%s\" to scan rows for compression",
+			 "using index \"%s\" to scan rows for converting to columnstore",
 			 get_rel_name(matched_index_rel->rd_id));
 
 		index_scan = index_beginscan(in_rel, matched_index_rel, GetTransactionSnapshot(), 0, 0);
@@ -476,7 +484,7 @@ compress_chunk(Oid in_table, Oid out_table, int insert_options)
 			row_compressor_process_ordered_slot(&row_compressor, slot, mycid);
 			if ((++nrows_processed % report_reltuples) == 0)
 				elog(DEBUG2,
-					 "compressed " INT64_FORMAT " rows from \"%s\"",
+					 "converted " INT64_FORMAT " rows to columnstore from \"%s\"",
 					 nrows_processed,
 					 RelationGetRelationName(in_rel));
 		}
@@ -485,7 +493,7 @@ compress_chunk(Oid in_table, Oid out_table, int insert_options)
 			row_compressor_flush(&row_compressor, mycid, true);
 
 		elog(DEBUG1,
-			 "finished compressing " INT64_FORMAT " rows from \"%s\"",
+			 "finished converting " INT64_FORMAT " rows to columnstore from \"%s\"",
 			 nrows_processed,
 			 RelationGetRelationName(in_rel));
 
@@ -496,7 +504,7 @@ compress_chunk(Oid in_table, Oid out_table, int insert_options)
 	else
 	{
 		elog(ts_guc_debug_compression_path_info ? INFO : DEBUG1,
-			 "using tuplesort to scan rows from \"%s\" for compression",
+			 "using tuplesort to scan rows from \"%s\" for converting to columnstore",
 			 RelationGetRelationName(in_rel));
 
 		Tuplesortstate *sorted_rel = compress_chunk_sort_relation(settings, in_rel);
@@ -505,16 +513,67 @@ compress_chunk(Oid in_table, Oid out_table, int insert_options)
 	}
 
 	row_compressor_close(&row_compressor);
-	if (!ts_guc_enable_delete_after_compression)
+
+	if (ts_guc_enable_delete_after_compression)
 	{
-		DEBUG_WAITPOINT("compression_done_before_truncate_uncompressed");
-		truncate_relation(in_table);
-		DEBUG_WAITPOINT("compression_done_after_truncate_uncompressed");
+		ereport(NOTICE,
+				(errcode(ERRCODE_WARNING_DEPRECATED_FEATURE),
+				 errmsg("timescaledb.enable_delete_after_compression is deprecated and will be "
+						"removed in a future version. Please use "
+						"timescaledb.compress_truncate_behaviour instead.")));
+		delete_relation_rows(in_table);
+		DEBUG_WAITPOINT("compression_done_after_delete_uncompressed");
 	}
 	else
 	{
-		delete_relation_rows(in_table);
-		DEBUG_WAITPOINT("compression_done_after_delete_uncompressed");
+		int lock_retry = 0;
+		switch (ts_guc_compress_truncate_behaviour)
+		{
+			case COMPRESS_TRUNCATE_ONLY:
+				DEBUG_WAITPOINT("compression_done_before_truncate_uncompressed");
+				truncate_relation(in_table);
+				DEBUG_WAITPOINT("compression_done_after_truncate_uncompressed");
+				break;
+			case COMPRESS_TRUNCATE_OR_DELETE:
+				DEBUG_WAITPOINT("compression_done_before_truncate_or_delete_uncompressed");
+				while (true)
+				{
+					if (ConditionalLockRelation(in_rel, AccessExclusiveLock))
+					{
+						truncate_relation(in_table);
+						break;
+					}
+
+					/*
+					 * Check for interrupts while trying to (re-)acquire the exclusive
+					 * lock.
+					 */
+					CHECK_FOR_INTERRUPTS();
+
+					if (++lock_retry >
+						(COMPRESS_TRUNCATE_LOCK_TIMEOUT / COMPRESS_TRUNCATE_LOCK_WAIT_INTERVAL))
+					{
+						/*
+						 * We failed to establish the lock in the specified number of
+						 * retries. This means we give up truncating and fallback to delete
+						 */
+						delete_relation_rows(in_table);
+						break;
+					}
+
+					(void) WaitLatch(MyLatch,
+									 WL_LATCH_SET | WL_TIMEOUT | WL_EXIT_ON_PM_DEATH,
+									 COMPRESS_TRUNCATE_LOCK_WAIT_INTERVAL,
+									 WAIT_EVENT_VACUUM_TRUNCATE);
+					ResetLatch(MyLatch);
+				}
+				DEBUG_WAITPOINT("compression_done_after_truncate_or_delete_uncompressed");
+				break;
+			case COMPRESS_TRUNCATE_DISABLED:
+				delete_relation_rows(in_table);
+				DEBUG_WAITPOINT("compression_done_after_delete_uncompressed");
+				break;
+		}
 	}
 
 	table_close(out_rel, NoLock);
@@ -770,6 +829,19 @@ build_column_map(const CompressionSettings *settings, Relation uncompressed_tabl
 			Ensure(!is_orderby || batch_minmax_builder != NULL,
 				   "orderby columns must have minmax metadata");
 
+			const AttrNumber bloom_attr_number =
+				compressed_column_metadata_attno(settings,
+												 uncompressed_table->rd_id,
+												 attr->attnum,
+												 compressed_table->rd_id,
+												 "bloom1");
+			if (AttributeNumberIsValid(bloom_attr_number))
+			{
+				const int bloom_attr_offset = AttrNumberGetAttrOffset(bloom_attr_number);
+				batch_minmax_builder =
+					batch_metadata_builder_bloom1_create(attr->atttypid, bloom_attr_offset);
+			}
+
 			*column = (PerColumn){
 				.compressor = compressor_for_type(attr->atttypid),
 				.metadata_builder = batch_minmax_builder,
@@ -793,6 +865,19 @@ build_column_map(const CompressionSettings *settings, Relation uncompressed_tabl
 	*pmap = map;
 }
 
+/* Check if we contain any compressors which need allocation limit checking */
+static bool
+check_for_limited_size_compressors(PerColumn *pcolumns, int16 natts)
+{
+	for (int i = 0; i < natts; i++)
+	{
+		if (pcolumns[i].compressor && pcolumns[i].compressor->is_full)
+			return true;
+	}
+
+	return false;
+}
+
 /********************
  ** row_compressor **
  ********************/
@@ -807,7 +892,7 @@ row_compressor_init(const CompressionSettings *settings, RowCompressor *row_comp
 		get_attnum(compressed_table->rd_id, NameStr(*count_metadata_name));
 	if (count_metadata_column_num == InvalidAttrNumber)
 		elog(ERROR,
-			 "missing metadata column '%s' in compressed table",
+			 "missing metadata column '%s' in columnstore table",
 			 COMPRESSION_COLUMN_METADATA_COUNT_NAME);
 
 	*row_compressor = (RowCompressor){
@@ -836,6 +921,12 @@ row_compressor_init(const CompressionSettings *settings, RowCompressor *row_comp
 					 &row_compressor->per_column,
 					 &row_compressor->uncompressed_col_to_compressed_col);
 
+	/* If we have dictionary or array compressors, we have to check compressor size so we don't end
+	 * up going over allocation limit */
+	row_compressor->needs_fullness_check =
+		check_for_limited_size_compressors(row_compressor->per_column,
+										   row_compressor->n_input_columns);
+
 	row_compressor->index_oid = get_compressed_chunk_index(row_compressor->resultRelInfo, settings);
 }
 
@@ -881,6 +972,42 @@ row_compressor_append_sorted_rows(RowCompressor *row_compressor, Tuplesortstate
 	ExecDropSingleTupleTableSlot(slot);
 }
 
+static bool
+row_compressor_is_full(RowCompressor *row_compressor, TupleTableSlot *row)
+{
+	if (row_compressor->rows_compressed_into_current_value >=
+		(uint32) ts_guc_compression_batch_size_limit)
+		return true;
+
+	if (!ts_guc_compression_enable_compressor_batch_limit)
+		return false;
+
+	if (!row_compressor->needs_fullness_check)
+		return false;
+
+	/* Check with every column compressor if they can add the next value to current batch */
+	int col;
+	for (col = 0; col < row_compressor->n_input_columns; col++)
+	{
+		Compressor *compressor = row_compressor->per_column[col].compressor;
+		bool is_null;
+		Datum val;
+
+		/* No compressor or the compressor has no check, just skip */
+		if (compressor == NULL || compressor->is_full == NULL)
+			continue;
+
+		val = slot_getattr(row, AttrOffsetGetAttrNumber(col), &is_null);
+		if (!is_null)
+		{
+			if (compressor->is_full(compressor, val))
+				return true;
+		}
+	}
+
+	return false;
+}
+
 static void
 row_compressor_process_ordered_slot(RowCompressor *row_compressor, TupleTableSlot *slot,
 									CommandId mycid)
@@ -894,8 +1021,7 @@ row_compressor_process_ordered_slot(RowCompressor *row_compressor, TupleTableSlo
 		row_compressor->first_iteration = false;
 	}
 	bool changed_groups = row_compressor_new_row_is_in_new_group(row_compressor, slot);
-	bool compressed_row_is_full = row_compressor->rows_compressed_into_current_value >=
-								  (uint32) ts_guc_compression_batch_size_limit;
+	bool compressed_row_is_full = row_compressor_is_full(row_compressor, slot);
 	if (compressed_row_is_full || changed_groups)
 	{
 		if (row_compressor->rows_compressed_into_current_value > 0)
@@ -1007,6 +1133,9 @@ row_compressor_append_row(RowCompressor *row_compressor, TupleTableSlot *row)
 static void
 row_compressor_flush(RowCompressor *row_compressor, CommandId mycid, bool changed_groups)
 {
+	MemoryContext old_ctx;
+	old_ctx = MemoryContextSwitchTo(row_compressor->per_row_ctx);
+
 	HeapTuple compressed_tuple;
 
 	for (int col = 0; col < row_compressor->n_input_columns; col++)
@@ -1112,6 +1241,7 @@ row_compressor_flush(RowCompressor *row_compressor, CommandId mycid, bool change
 	row_compressor->rows_compressed_into_current_value = 0;
 
 	MemoryContextReset(row_compressor->per_row_ctx);
+	MemoryContextSwitchTo(old_ctx);
 }
 
 void
@@ -1392,18 +1522,31 @@ create_per_compressed_column(RowDecompressor *decompressor)
 	}
 }
 
-/*
- * Decompresses the current compressed batch into decompressed_slots, and returns
- * the number of rows in batch.
- */
-int
-decompress_batch(RowDecompressor *decompressor)
+static void
+init_iterator(RowDecompressor *decompressor, CompressedDataHeader *header, int input_column)
 {
-	if (decompressor->unprocessed_tuples)
-		return decompressor->unprocessed_tuples;
+	Assert(decompressor->num_compressed_columns > input_column);
+	PerCompressedColumn *column_info = &decompressor->per_compressed_cols[input_column];
 
-	MemoryContext old_ctx = MemoryContextSwitchTo(decompressor->per_compressed_row_ctx);
+	/* Special compression block with the NULL compression algorithm,
+	 * tells that all values in the compressed block are NULLs.
+	 */
+	if (header->compression_algorithm == COMPRESSION_ALGORITHM_NULL)
+	{
+		column_info->iterator = NULL;
+		decompressor->compressed_is_nulls[input_column] = true;
+		decompressor->decompressed_is_nulls[column_info->decompressed_column_offset] = true;
+		return;
+	}
 
+	column_info->iterator =
+		definitions[header->compression_algorithm]
+			.iterator_init_forward(PointerGetDatum(header), column_info->decompressed_type);
+}
+
+static void
+init_batch(RowDecompressor *decompressor, AttrNumber *attnos, int num_attnos)
+{
 	/*
 	 * Set segmentbys and compressed columns with default value.
 	 */
@@ -1440,6 +1583,23 @@ decompress_batch(RowDecompressor *decompressor)
 			continue;
 		}
 
+		/* Only initialize required columns if specified. */
+		bool found = num_attnos == 0;
+		for (int i = 0; i < num_attnos; i++)
+		{
+			if (output_index == AttrNumberGetAttrOffset(attnos[i]))
+			{
+				found = true;
+				break;
+			}
+		}
+
+		if (!found)
+		{
+			column_info->iterator = NULL;
+			continue;
+		}
+
 		/* Normal compressed column. */
 		Datum compressed_datum = PointerGetDatum(
 			detoaster_detoast_attr_copy((struct varlena *) DatumGetPointer(
@@ -1448,21 +1608,23 @@ decompress_batch(RowDecompressor *decompressor)
 										CurrentMemoryContext));
 		CompressedDataHeader *header = get_compressed_data_header(compressed_datum);
 
-		/* Special compression block with the NULL compression algorithm,
-		 * tells that all values in the compressed block are NULLs.
-		 */
-		if (header->compression_algorithm == COMPRESSION_ALGORITHM_NULL)
-		{
-			column_info->iterator = NULL;
-			decompressor->compressed_is_nulls[input_column] = true;
-			decompressor->decompressed_is_nulls[output_index] = true;
-			continue;
-		}
-
-		column_info->iterator =
-			definitions[header->compression_algorithm]
-				.iterator_init_forward(PointerGetDatum(header), column_info->decompressed_type);
+		init_iterator(decompressor, header, input_column);
 	}
+}
+
+/*
+ * Decompresses the current compressed batch into decompressed_slots, and returns
+ * the number of rows in batch.
+ */
+int
+decompress_batch(RowDecompressor *decompressor)
+{
+	if (decompressor->unprocessed_tuples)
+		return decompressor->unprocessed_tuples;
+
+	MemoryContext old_ctx = MemoryContextSwitchTo(decompressor->per_compressed_row_ctx);
+
+	init_batch(decompressor, NULL, 0);
 
 	/*
 	 * Set the number of batch rows from count metadata column.
@@ -1544,6 +1706,126 @@ decompress_batch(RowDecompressor *decompressor)
 	return n_batch_rows;
 }
 
+/*
+ * Decompresses a single row from current compressed batch
+ * into decompressed_values and decompressed_is_nulls based on the
+ * attnos provided.
+ *
+ * Returns true if the row was decompressed or false if it finished the batch.
+ */
+bool
+decompress_batch_next_row(RowDecompressor *decompressor, AttrNumber *attnos, int num_attnos)
+{
+	MemoryContext old_ctx = MemoryContextSwitchTo(decompressor->per_compressed_row_ctx);
+
+	if (decompressor->unprocessed_tuples > 0)
+	{
+		decompressor->unprocessed_tuples--;
+		if (decompressor->unprocessed_tuples == 0)
+		{
+			MemoryContextSwitchTo(old_ctx);
+			return false;
+		}
+	}
+	else
+	{
+		decompressor->batches_decompressed++;
+		init_batch(decompressor, attnos, num_attnos);
+
+		/*
+		 * Set the number of batch rows from count metadata column.
+		 */
+		decompressor->unprocessed_tuples =
+			DatumGetInt32(decompressor->compressed_datums[decompressor->count_compressed_attindex]);
+		CheckCompressedData(decompressor->unprocessed_tuples > 0);
+		CheckCompressedData(decompressor->unprocessed_tuples <= GLOBAL_MAX_ROWS_PER_COMPRESSION);
+	}
+
+	for (int16 col = 0; col < decompressor->num_compressed_columns; col++)
+	{
+		PerCompressedColumn *column_info = &decompressor->per_compressed_cols[col];
+		if (column_info->iterator == NULL)
+		{
+			continue;
+		}
+		Assert(column_info->is_compressed);
+
+		const int output_index = column_info->decompressed_column_offset;
+		const DecompressResult value = column_info->iterator->try_next(column_info->iterator);
+		Assert(!value.is_done);
+		decompressor->decompressed_datums[output_index] = value.val;
+		decompressor->decompressed_is_nulls[output_index] = value.is_null;
+	}
+
+	decompressor->tuples_decompressed++;
+
+	MemoryContextSwitchTo(old_ctx);
+
+	return true;
+}
+
+/* Decompress single column using vectorized decompression */
+ArrowArray *
+decompress_single_column(RowDecompressor *decompressor, AttrNumber attno, bool *single_value)
+{
+	int16 target_col = -1;
+	PerCompressedColumn *column_info = NULL;
+
+	for (int16 col = 0; col < decompressor->num_compressed_columns; col++)
+	{
+		column_info = &decompressor->per_compressed_cols[col];
+		if (!column_info->is_compressed)
+			continue;
+
+		if (column_info->decompressed_column_offset == AttrNumberGetAttrOffset(attno))
+		{
+			target_col = col;
+			break;
+		}
+	}
+	Assert(column_info && target_col > -1);
+
+	if (decompressor->compressed_is_nulls[target_col])
+	{
+		/* Compressed column has a default value, handle it by generating
+		 * a single-value ArrowArray based on the default value. This will have to
+		 * be handled specially because of the assumption that the whole row has
+		 * this default value.
+		 */
+		*single_value = true;
+		bool isnull;
+		Datum default_datum = getmissingattr(decompressor->out_desc, attno, &isnull);
+
+		return make_single_value_arrow(column_info->decompressed_type, default_datum, isnull);
+	}
+
+	*single_value = false;
+
+	Datum compressed_datum = PointerGetDatum(
+		detoaster_detoast_attr_copy((struct varlena *) DatumGetPointer(
+										decompressor->compressed_datums[target_col]),
+									&decompressor->detoaster,
+									CurrentMemoryContext));
+	CompressedDataHeader *header = get_compressed_data_header(compressed_datum);
+
+	/* Handle NULL compression algorithm */
+	if (header->compression_algorithm == COMPRESSION_ALGORITHM_NULL)
+	{
+		*single_value = true;
+		return make_single_value_arrow(column_info->decompressed_type, (Datum) NULL, true);
+	}
+
+	DecompressAllFunction decompress_all =
+		tsl_get_decompress_all_function(header->compression_algorithm,
+										column_info->decompressed_type);
+
+	Assert(decompress_all);
+
+	return decompress_all(compressed_datum,
+						  column_info->decompressed_type,
+						  decompressor->per_compressed_row_ctx);
+}
+
 int
 row_decompressor_decompress_row_to_table(RowDecompressor *decompressor)
 {
@@ -1740,14 +2022,7 @@ tsl_compressed_data_recv(PG_FUNCTION_ARGS)
 	if (header.compression_algorithm >= _END_COMPRESSION_ALGORITHMS)
 		elog(ERROR, "invalid compression algorithm %d", header.compression_algorithm);
 
-	if (header.compression_algorithm == COMPRESSION_ALGORITHM_NULL)
-	{
-		PG_RETURN_NULL();
-	}
-	else
-	{
-		return definitions[header.compression_algorithm].compressed_data_recv(buf);
-	}
+	return definitions[header.compression_algorithm].compressed_data_recv(buf);
 }
 
 extern Datum
