@@ -21,6 +21,7 @@
 #include <commands/tablecmds.h>
 #include <commands/tablespace.h>
 #include <common/md5.h>
+#include <executor/spi.h>
 #include <miscadmin.h>
 #include <nodes/makefuncs.h>
 #include <parser/parse_type.h>
@@ -39,7 +40,7 @@
 #include "chunk_index.h"
 #include "compression.h"
 #include "compression/compression_storage.h"
-#include "compression_with_clause.h"
+#include "compression/sparse_index_bloom1.h"
 #include "create.h"
 #include "custom_type_cache.h"
 #include "guc.h"
@@ -50,9 +51,10 @@
 #include "ts_catalog/compression_settings.h"
 #include "ts_catalog/continuous_agg.h"
 #include "utils.h"
-#include <executor/spi.h>
+#include "with_clause/alter_table_with_clause.h"
+#include "with_clause/create_table_with_clause.h"
 
-static const char *sparse_index_types[] = { "min", "max" };
+static const char *sparse_index_types[] = { "min", "max", "bloom1" };
 
 #ifdef USE_ASSERT_CHECKING
 static bool
@@ -71,7 +73,7 @@ is_sparse_index_type(const char *type)
 #endif
 
 static void validate_hypertable_for_compression(Hypertable *ht);
-static List *build_columndefs(CompressionSettings *settings, Oid src_relid);
+static List *build_columndefs(CompressionSettings *settings, Oid src_reloid);
 static ColumnDef *build_columndef_singlecolumn(const char *colname, Oid typid);
 static void compression_settings_update(Hypertable *ht, CompressionSettings *settings,
 										WithClauseResult *with_clause_options);
@@ -132,7 +134,7 @@ compressed_column_metadata_name_v2(const char *metadata_type, const char *column
 	{
 		const char *errstr = NULL;
 		char hash[33];
-		Ensure(pg_md5_hash_compat(column_name, len, hash, &errstr), "md5 computation failure");
+		Ensure(pg_md5_hash(column_name, len, hash, &errstr), "md5 computation failure");
 
 		result = psprintf("_ts_meta_v2_%.6s_%.4s_%.39s", metadata_type, hash, column_name);
 	}
@@ -153,7 +155,8 @@ compressed_column_metadata_attno(const CompressionSettings *settings, Oid chunk_
 	char *attname = get_attname(chunk_reloid, chunk_attno, /* missing_ok = */ false);
 	int16 orderby_pos = ts_array_position(settings->fd.orderby, attname);
 
-	if (orderby_pos != 0)
+	if (orderby_pos != 0 &&
+		(strcmp(metadata_type, "min") == 0 || strcmp(metadata_type, "max") == 0))
 	{
 		char *metadata_name = compression_column_segment_metadata_name(metadata_type, orderby_pos);
 		return get_attnum(compressed_reloid, metadata_name);
@@ -163,6 +166,76 @@ compressed_column_metadata_attno(const CompressionSettings *settings, Oid chunk_
 	return get_attnum(compressed_reloid, metadata_name);
 }
 
+/*
+ * The heuristic for whether we should use the bloom filter sparse index.
+ */
+static bool
+should_create_bloom_sparse_index(Form_pg_attribute attr, TypeCacheEntry *type, Oid src_reloid)
+{
+	/*
+	 * The index must be enabled by the GUC.
+	 */
+	if (!ts_guc_enable_sparse_index_bloom)
+	{
+		return false;
+	}
+
+	/*
+	 * The type must be hashable. For some types we use our own hash functions
+	 * which have better characteristics.
+	 */
+	FmgrInfo *finfo = NULL;
+	if (bloom1_get_hash_function(attr->atttypid, &finfo) == NULL)
+	{
+		return false;
+	}
+
+	/*
+	 * For time types, we expect:
+	 * 1) range queries, not equality,
+	 * 2) correlation with the orderby columns, e.g. creation time correlates
+	 *    with the update time that is used as orderby.
+	 * This makes minmax indexes more suitable than bloom filters.
+	 */
+	if (attr->atttypid == TIMESTAMPTZOID || attr->atttypid == TIMESTAMPOID ||
+		attr->atttypid == TIMEOID || attr->atttypid == TIMETZOID || attr->atttypid == DATEOID)
+	{
+		return false;
+	}
+
+	/*
+	 * For fractional arithmetic types, equality queries are unlikely.
+	 */
+	if (attr->atttypid == FLOAT4OID || attr->atttypid == FLOAT8OID || attr->atttypid == NUMERICOID)
+	{
+		return false;
+	}
+
+	/*
+	 * Bloom filters for 1k elements with 2% false positive rate require about
+	 * one byte per element, so there's no point in using them for smaller data
+	 * types that typically compress to less than that.
+	 */
+	if (type->typlen > 0 && type->typlen < 4)
+	{
+		return false;
+	}
+
+	/*
+	 * Bloom filter pushdown is not implemented for TAM at the moment, so keep
+	 * the old behavior with minmax sparse indexes. This check is actually not
+	 * enough, because the compressed chunk table is created when the
+	 * uncompressed chunk table is converted to TAM, and at this time it still
+	 * has the normal heap access method.
+	 */
+	if (ts_is_hypercore_am(ts_get_rel_am(src_reloid)))
+	{
+		return false;
+	}
+
+	return true;
+}
+
 /*
  * return the columndef list for compressed hypertable.
  * we do this by getting the source hypertable's attrs,
@@ -172,16 +245,16 @@ compressed_column_metadata_attno(const CompressionSettings *settings, Oid chunk_
  *     all other cols have COMPRESSEDDATA_TYPE type
  */
 static List *
-build_columndefs(CompressionSettings *settings, Oid src_relid)
+build_columndefs(CompressionSettings *settings, Oid src_reloid)
 {
 	Oid compresseddata_oid = ts_custom_type_cache_get(CUSTOM_TYPE_COMPRESSED_DATA)->type_oid;
 	ArrayType *segmentby = settings->fd.segmentby;
 	List *compressed_column_defs = NIL;
 	List *segmentby_column_defs = NIL;
 
-	Relation rel = table_open(src_relid, AccessShareLock);
+	Relation rel = table_open(src_reloid, AccessShareLock);
 
-	Bitmapset *btree_columns = NULL;
+	Bitmapset *index_columns = NULL;
 	if (ts_guc_auto_sparse_indexes)
 	{
 		/*
@@ -202,23 +275,27 @@ build_columndefs(CompressionSettings *settings, Oid src_relid)
 			 * kinds of queries as the uncompressed index. The simplest case is btree
 			 * which can satisfy equality and comparison tests, same as sparse minmax.
 			 *
-			 * We can be smarter here, e.g. for 'BRIN', sparse minmax can be similar
-			 * to 'BRIN' with range opclass, but not for bloom filter opclass. For GIN,
-			 * sparse minmax is useless because it doesn't help satisfy text search
-			 * queries, and so on. Currently we check only the simplest btree case.
+			 * If an uncompressed column has an index, we want to create a
+			 * sparse index for it as well. A sparse index can't satisfy ordering
+			 * queries, but at least we can use a bloom index to satisfy equality
+			 * queries. Create it when we have uncompressed index types that can
+			 * also satisfy equality.
 			 */
-			if (index_info->ii_Am != BTREE_AM_OID)
+			if (index_info->ii_Am != BTREE_AM_OID && index_info->ii_Am != HASH_AM_OID &&
+				index_info->ii_Am != BRIN_AM_OID)
 			{
 				continue;
 			}
 
 			for (int i = 0; i < index_info->ii_NumIndexKeyAttrs; i++)
 			{
-				AttrNumber attno = index_info->ii_IndexAttrNumbers[i];
-				if (attno != InvalidAttrNumber)
+				const AttrNumber attno = index_info->ii_IndexAttrNumbers[i];
+				if (attno == InvalidAttrNumber)
 				{
-					btree_columns = bms_add_member(btree_columns, attno);
+					continue;
 				}
+
+				index_columns = bms_add_member(index_columns, attno);
 			}
 		}
 	}
@@ -254,7 +331,7 @@ build_columndefs(CompressionSettings *settings, Oid src_relid)
 		 * respective compressed column, because they are accessed before
 		 * decompression.
 		 */
-		bool is_orderby = ts_array_is_member(settings->fd.orderby, NameStr(attr->attname));
+		const bool is_orderby = ts_array_is_member(settings->fd.orderby, NameStr(attr->attname));
 		if (is_orderby)
 		{
 			int index = ts_array_position(settings->fd.orderby, NameStr(attr->attname));
@@ -271,49 +348,78 @@ build_columndefs(CompressionSettings *settings, Oid src_relid)
 						 errdetail("Could not identify a less-than operator for the type.")));
 
 			/* segment_meta min and max columns */
-			compressed_column_defs = lappend(compressed_column_defs,
-											 makeColumnDef(column_segment_min_name(index),
-														   attr->atttypid,
-														   attr->atttypmod,
-														   attr->attcollation));
-			compressed_column_defs = lappend(compressed_column_defs,
-											 makeColumnDef(column_segment_max_name(index),
-														   attr->atttypid,
-														   attr->atttypmod,
-														   attr->attcollation));
+			ColumnDef *def = makeColumnDef(column_segment_min_name(index),
+										   attr->atttypid,
+										   attr->atttypmod,
+										   attr->attcollation);
+			def->storage = TYPSTORAGE_PLAIN;
+			compressed_column_defs = lappend(compressed_column_defs, def);
+			def = makeColumnDef(column_segment_max_name(index),
+								attr->atttypid,
+								attr->atttypmod,
+								attr->attcollation);
+			def->storage = TYPSTORAGE_PLAIN;
+			compressed_column_defs = lappend(compressed_column_defs, def);
 		}
-		else if (bms_is_member(attr->attnum, btree_columns))
+		else if (bms_is_member(attr->attnum, index_columns))
 		{
-			TypeCacheEntry *type = lookup_type_cache(attr->atttypid, TYPECACHE_LT_OPR);
+			TypeCacheEntry *type =
+				lookup_type_cache(attr->atttypid, TYPECACHE_LT_OPR | TYPECACHE_HASH_EXTENDED_PROC);
 
-			if (OidIsValid(type->lt_opr))
+			/*
+			 * We can have various unusual user-defined types which do not
+			 * support comparison or hashing. The sparse indexes for the
+			 * non-orderby columns are not required for correctness, so just
+			 * don't create the sparse index if we lack the suitable operators.
+			 */
+			bool can_use_minmax = OidIsValid(type->lt_opr);
+
+			if (should_create_bloom_sparse_index(attr, type, src_reloid))
 			{
 				/*
-				 * Here we create minmax metadata for the columns for which
-				 * we have btree indexes. Not sure it is technically possible
-				 * to have a btree index for a column and at the same time
-				 * not have a "less" operator for it. Still, we can have
-				 * various unusual user-defined types, and the minmax metadata
-				 * for the rest of the columns are not required for correctness,
-				 * so play it safe and just don't create the metadata if we don't
-				 * have an operator.
+				 * Add bloom filter sparse index for this column.
 				 */
-				compressed_column_defs =
-					lappend(compressed_column_defs,
-							makeColumnDef(compressed_column_metadata_name_v2("min",
-																			 NameStr(
-																				 attr->attname)),
-										  attr->atttypid,
-										  attr->atttypmod,
-										  attr->attcollation));
-				compressed_column_defs =
-					lappend(compressed_column_defs,
-							makeColumnDef(compressed_column_metadata_name_v2("max",
-																			 NameStr(
-																				 attr->attname)),
-										  attr->atttypid,
-										  attr->atttypmod,
-										  attr->attcollation));
+				ColumnDef *bloom_column_def =
+					makeColumnDef(compressed_column_metadata_name_v2("bloom1",
+																	 NameStr(attr->attname)),
+								  ts_custom_type_cache_get(CUSTOM_TYPE_BLOOM1)->type_oid,
+								  /* typmod = */ -1,
+								  /* collation = */ 0);
+
+				/*
+				 * We have our custom compression for bloom filters, and the
+				 * result is almost incompressible with lz4 (~2%), so disable it.
+				 */
+				bloom_column_def->storage = TYPSTORAGE_EXTERNAL;
+
+				compressed_column_defs = lappend(compressed_column_defs, bloom_column_def);
+			}
+			else if (can_use_minmax)
+			{
+				/*
+				 * Add minmax sparse index for this column.
+				 */
+				ColumnDef *def =
+					makeColumnDef(compressed_column_metadata_name_v2("min", NameStr(attr->attname)),
+								  attr->atttypid,
+								  attr->atttypmod,
+								  attr->attcollation);
+				if (attr->attstorage != TYPSTORAGE_PLAIN)
+				{
+					def->storage = TYPSTORAGE_MAIN;
+				}
+				compressed_column_defs = lappend(compressed_column_defs, def);
+
+				def =
+					makeColumnDef(compressed_column_metadata_name_v2("max", NameStr(attr->attname)),
+								  attr->atttypid,
+								  attr->atttypmod,
+								  attr->attcollation);
+				if (attr->attstorage != TYPSTORAGE_PLAIN)
+				{
+					def->storage = TYPSTORAGE_MAIN;
+				}
+				compressed_column_defs = lappend(compressed_column_defs, def);
 			}
 		}
 
@@ -442,6 +548,26 @@ create_compress_chunk(Hypertable *compress_ht, Chunk *src_chunk, Oid table_id)
 	tablespace_oid = get_rel_tablespace(src_chunk->table_id);
 	CompressionSettings *settings = ts_compression_settings_get(src_chunk->hypertable_relid);
 
+	/*
+	 * On hypertables created with CREATE TABLE ... WITH we enable compression
+	 * by default but do not create CompressionSettings immediately assuming
+	 * that we have more information available when the first compression
+	 * is actually triggered allowing us to generate better compression
+	 * settings.
+	 */
+	if (!settings)
+	{
+		settings = ts_compression_settings_create(src_chunk->hypertable_relid,
+												  InvalidOid,
+												  NULL,
+												  NULL,
+												  NULL,
+												  NULL);
+
+		Hypertable *ht = ts_hypertable_get_by_id(src_chunk->fd.hypertable_id);
+		compression_settings_update(ht, settings, ts_alter_table_with_clause_parse(NIL));
+	}
+
 	if (OidIsValid(table_id))
 		compress_chunk->table_id = table_id;
 	else
@@ -452,7 +578,7 @@ create_compress_chunk(Hypertable *compress_ht, Chunk *src_chunk, Oid table_id)
 	}
 
 	if (!OidIsValid(compress_chunk->table_id))
-		elog(ERROR, "could not create compressed chunk table");
+		elog(ERROR, "could not create columnstore chunk table");
 
 	/* Materialize current compression settings for this chunk */
 	ts_compression_settings_materialize(settings, src_chunk->table_id, compress_chunk->table_id);
@@ -546,10 +672,10 @@ validate_existing_constraints(Hypertable *ht, CompressionSettings *settings)
 		{
 			ereport(ERROR,
 					(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-					 errmsg("constraint %s is not supported for compression",
+					 errmsg("constraint %s is not supported for converting to columnstore",
 							NameStr(form->conname)),
 					 errhint("Exclusion constraints are not supported on hypertables that are "
-							 "compressed.")));
+							 "converted to columnstore.")));
 		}
 		else
 		{
@@ -652,15 +778,15 @@ drop_existing_compression_table(Hypertable *ht)
 	if (ts_chunk_exists_with_compression(ht->fd.id))
 		ereport(ERROR,
 				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-				 errmsg("cannot drop compression hypertable with compressed chunks")));
+				 errmsg("cannot drop columnstore-enabled hypertable with columnstore chunks")));
 
 	Hypertable *compressed = ts_hypertable_get_by_id(ht->fd.compressed_hypertable_id);
 	if (compressed == NULL)
 		ereport(ERROR,
 				(errcode(ERRCODE_INTERNAL_ERROR),
-				 errmsg("compressed hypertable not found"),
-				 errdetail("compression was enabled on \"%s\", but its internal"
-						   " compressed hypertable could not be found.",
+				 errmsg("columnstore-enabled hypertable not found"),
+				 errdetail("columnstore was enabled on \"%s\", but its internal"
+						   " columnstore hypertable could not be found.",
 						   NameStr(ht->fd.table_name))));
 
 	/* need to drop the old compressed hypertable in case the segment by columns changed (and
@@ -679,7 +805,7 @@ disable_compression(Hypertable *ht, WithClauseResult *with_clause_options)
 	if (ts_chunk_exists_with_compression(ht->fd.id))
 		ereport(ERROR,
 				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-				 errmsg("cannot disable compression on hypertable with compressed chunks")));
+				 errmsg("cannot disable columnstore on hypertable with columnstore chunks")));
 
 	if (TS_HYPERTABLE_HAS_COMPRESSION_TABLE(ht))
 		drop_existing_compression_table(ht);
@@ -763,12 +889,11 @@ update_compress_chunk_time_interval(Hypertable *ht, WithClauseResult *with_claus
  * 4. Copy constraints to internal compression table
  */
 bool
-tsl_process_compress_table(AlterTableCmd *cmd, Hypertable *ht,
-						   WithClauseResult *with_clause_options)
+tsl_process_compress_table(Hypertable *ht, WithClauseResult *with_clause_options)
 {
 	int32 compress_htid;
-	bool compress_disable = !with_clause_options[CompressEnabled].is_default &&
-							!DatumGetBool(with_clause_options[CompressEnabled].parsed);
+	bool compress_disable = !with_clause_options[AlterTableFlagCompress].is_default &&
+							!DatumGetBool(with_clause_options[AlterTableFlagCompress].parsed);
 	CompressionSettings *settings;
 
 	ts_feature_flag_check(FEATURE_HYPERTABLE_COMPRESSION);
@@ -848,14 +973,14 @@ validate_hypertable_for_compression(Hypertable *ht)
 	{
 		ereport(ERROR,
 				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-				 errmsg("cannot compress internal compression hypertable")));
+				 errmsg("cannot compress internal columnstore hypertable")));
 	}
 
 	/*check row security settings for the table */
 	if (ts_has_row_security(ht->main_table_relid))
 		ereport(ERROR,
 				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-				 errmsg("compression cannot be used on table with row security")));
+				 errmsg("columnstore cannot be used on table with row security")));
 
 	Relation rel = table_open(ht->main_table_relid, AccessShareLock);
 	TupleDesc tupdesc = RelationGetDescr(rel);
@@ -882,7 +1007,7 @@ validate_hypertable_for_compression(Hypertable *ht)
 					COMPRESSION_COLUMN_METADATA_PREFIX,
 					strlen(COMPRESSION_COLUMN_METADATA_PREFIX)) == 0)
 			elog(ERROR,
-				 "cannot compress tables with reserved column prefix '%s'",
+				 "cannot convert tables with reserved column prefix '%s' to columnstore",
 				 COMPRESSION_COLUMN_METADATA_PREFIX);
 	}
 
@@ -890,8 +1015,10 @@ validate_hypertable_for_compression(Hypertable *ht)
 	{
 		ereport(WARNING,
 				(errmsg("compressed row size might exceed maximum row size"),
-				 errdetail("Estimated row size of compressed hypertable is %zu. This exceeds the "
-						   "maximum size of %zu and can cause compression of chunks to fail.",
+				 errdetail("Estimated row size of columnstore-enabled hypertable is %zu. This "
+						   "exceeds the "
+						   "maximum size of %zu and can cause conversion of chunks to columnstore "
+						   "to fail.",
 						   row_size,
 						   MaxHeapTupleSize)));
 	}
@@ -1172,30 +1299,34 @@ compression_settings_update(Hypertable *ht, CompressionSettings *settings,
 		(settings->fd.orderby && settings->fd.orderby_desc && settings->fd.orderby_nullsfirst) ||
 		(!settings->fd.orderby && !settings->fd.orderby_desc && !settings->fd.orderby_nullsfirst));
 
-	if (!with_clause_options[CompressChunkTimeInterval].is_default)
+	if (!with_clause_options[AlterTableFlagCompressChunkTimeInterval].is_default)
 	{
 		update_compress_chunk_time_interval(ht, with_clause_options);
 	}
 
-	if (!with_clause_options[CompressSegmentBy].is_default)
+	if (!with_clause_options[AlterTableFlagSegmentBy].is_default)
 	{
-		settings->fd.segmentby = ts_compress_hypertable_parse_segment_by(with_clause_options, ht);
+		settings->fd.segmentby =
+			ts_compress_hypertable_parse_segment_by(with_clause_options[AlterTableFlagSegmentBy],
+													ht);
 	}
-	else if (!settings->fd.segmentby)
+	else if (!settings->fd.segmentby && !settings->fd.orderby &&
+			 with_clause_options[AlterTableFlagOrderBy].is_default)
 	{
 		settings->fd.segmentby = compression_setting_segmentby_get_default(ht);
 	}
 
-	if (!with_clause_options[CompressOrderBy].is_default || !settings->fd.orderby)
+	if (!with_clause_options[AlterTableFlagOrderBy].is_default || !settings->fd.orderby)
 	{
 		OrderBySettings obs;
-		if (with_clause_options[CompressOrderBy].is_default)
+		if (with_clause_options[AlterTableFlagOrderBy].is_default)
 		{
 			obs = compression_setting_orderby_get_default(ht, settings->fd.segmentby);
 		}
 		else
 		{
-			obs = ts_compress_hypertable_parse_order_by(with_clause_options, ht);
+			obs = ts_compress_hypertable_parse_order_by(with_clause_options[AlterTableFlagOrderBy],
+														ht);
 			obs = add_time_to_order_by_if_not_included(obs, settings->fd.segmentby, ht);
 		}
 		settings->fd.orderby = obs.orderby;
@@ -1251,12 +1382,12 @@ tsl_process_compress_table_drop_column(Hypertable *ht, char *name)
 
 	CompressionSettings *settings = ts_compression_settings_get(ht->main_table_relid);
 
-	if (ts_array_is_member(settings->fd.segmentby, name) ||
-		ts_array_is_member(settings->fd.orderby, name))
+	if (settings && (ts_array_is_member(settings->fd.segmentby, name) ||
+					 ts_array_is_member(settings->fd.orderby, name)))
 		ereport(ERROR,
 				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 				 errmsg("cannot drop orderby or segmentby column from a hypertable with "
-						"compression enabled")));
+						"columnstore enabled")));
 
 	List *chunks = ts_chunk_get_by_hypertable_id(ht->fd.compressed_hypertable_id);
 	ListCell *lc;
@@ -1270,7 +1401,7 @@ tsl_process_compress_table_drop_column(Hypertable *ht, char *name)
 			ereport(ERROR,
 					(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
 					 errmsg("cannot drop orderby or segmentby column from a chunk with "
-							"compression enabled")));
+							"columnstore enabled")));
 	}
 
 	if (TS_HYPERTABLE_HAS_COMPRESSION_TABLE(ht))
@@ -1300,7 +1431,7 @@ tsl_process_compress_table_rename_column(Hypertable *ht, const RenameStmt *stmt)
 				COMPRESSION_COLUMN_METADATA_PREFIX,
 				strlen(COMPRESSION_COLUMN_METADATA_PREFIX)) == 0)
 		elog(ERROR,
-			 "cannot compress tables with reserved column prefix '%s'",
+			 "cannot convert tables with reserved column prefix '%s' to columnstore",
 			 COMPRESSION_COLUMN_METADATA_PREFIX);
 
 	if (!TS_HYPERTABLE_HAS_COMPRESSION_TABLE(ht))
@@ -1337,3 +1468,55 @@ tsl_process_compress_table_rename_column(Hypertable *ht, const RenameStmt *stmt)
 		}
 	}
 }
+
+/*
+ * Enables compression for a hypertable without creating initial configuration
+ */
+void
+tsl_compression_enable(Hypertable *ht, WithClauseResult *with_clause_options)
+{
+	LockRelationOid(catalog_get_table_id(ts_catalog_get(), HYPERTABLE), RowExclusiveLock);
+	Oid ownerid = ts_rel_get_owner(ht->main_table_relid);
+	Oid tablespace_oid = get_rel_tablespace(ht->main_table_relid);
+	if (!with_clause_options[CreateTableFlagOrderBy].is_default ||
+		!with_clause_options[CreateTableFlagSegmentBy].is_default)
+	{
+		CompressionSettings *settings = ts_compression_settings_create(ht->main_table_relid,
+																	   InvalidOid,
+																	   NULL,
+																	   NULL,
+																	   NULL,
+																	   NULL);
+
+		if (!with_clause_options[CreateTableFlagSegmentBy].is_default)
+		{
+			settings->fd.segmentby =
+				ts_compress_hypertable_parse_segment_by(with_clause_options
+															[CreateTableFlagSegmentBy],
+														ht);
+		}
+
+		if (!with_clause_options[CreateTableFlagOrderBy].is_default || !settings->fd.orderby)
+		{
+			OrderBySettings obs;
+			if (with_clause_options[CreateTableFlagOrderBy].is_default)
+			{
+				obs = compression_setting_orderby_get_default(ht, settings->fd.segmentby);
+			}
+			else
+			{
+				obs = ts_compress_hypertable_parse_order_by(with_clause_options
+																[CreateTableFlagOrderBy],
+															ht);
+				obs = add_time_to_order_by_if_not_included(obs, settings->fd.segmentby, ht);
+			}
+			settings->fd.orderby = obs.orderby;
+			settings->fd.orderby_desc = obs.orderby_desc;
+			settings->fd.orderby_nullsfirst = obs.orderby_nullsfirst;
+		}
+
+		ts_compression_settings_update(settings);
+	}
+	int compress_htid = compression_hypertable_create(ht, ownerid, tablespace_oid);
+	ts_hypertable_set_compressed(ht, compress_htid);
+}
