@@ -158,7 +158,14 @@ cache_inval_entry_init(ContinuousAggsCacheInvalEntry *cache_entry, int32 hyperta
 
 	cache_entry->hypertable_id = hypertable_id;
 	cache_entry->hypertable_relid = ht->main_table_relid;
-	cache_entry->hypertable_open_dimension = *hyperspace_get_open_dimension(ht->space, 0);
+
+	const Dimension *open_dim = hyperspace_get_open_dimension(ht->space, 0);
+	if (!open_dim)
+		ereport(ERROR,
+				(errcode(ERRCODE_OBJECT_NOT_IN_PREREQUISITE_STATE),
+				 errmsg("hypertable has no open partitioning dimension")));
+
+	cache_entry->hypertable_open_dimension = *open_dim;
 	if (cache_entry->hypertable_open_dimension.partitioning != NULL)
 	{
 		PartitioningInfo *open_dim_part_info =
@@ -170,7 +177,7 @@ cache_inval_entry_init(ContinuousAggsCacheInvalEntry *cache_entry, int32 hyperta
 	cache_entry->value_is_set = false;
 	cache_entry->lowest_modified_value = INVAL_POS_INFINITY;
 	cache_entry->greatest_modified_value = INVAL_NEG_INFINITY;
-	ts_cache_release(ht_cache);
+	ts_cache_release(&ht_cache);
 }
 
 static inline void
