@@ -23,4 +23,5 @@ extern void continuous_agg_refresh_internal(const ContinuousAgg *cagg,
 											bool force);
 extern List *continuous_agg_split_refresh_window(ContinuousAgg *cagg,
 												 InternalTimeRange *original_refresh_window,
-												 int32 buckets_per_batch);
+												 int32 buckets_per_batch,
+												 bool refresh_newest_first);
