@@ -299,7 +299,7 @@ build_compressed_scan_pathkeys(const SortInfo *sort_info, PlannerInfo *root, Lis
 			for (; lc != NULL; lc = lnext(chunk_pathkeys, lc))
 			{
 				pk = lfirst(lc);
-				expr = find_em_expr_for_rel(pk->pk_eclass, info->chunk_rel);
+				expr = ts_find_em_expr_for_rel(pk->pk_eclass, info->chunk_rel);
 
 				Assert(expr != NULL && IsA(expr, Var));
 				var = castNode(Var, expr);
@@ -443,12 +443,14 @@ static void
 cost_decompress_chunk(PlannerInfo *root, Path *path, Path *compressed_path)
 {
 	/* startup_cost is cost before fetching first tuple */
-	if (compressed_path->rows > 0)
-		path->startup_cost = compressed_path->total_cost / compressed_path->rows;
+	const double compressed_rows = Max(1, compressed_path->rows);
+	path->startup_cost =
+		compressed_path->startup_cost +
+		(compressed_path->total_cost - compressed_path->startup_cost) / compressed_rows;
 
 	/* total_cost is cost for fetching all tuples */
-	path->total_cost = compressed_path->total_cost + path->rows * cpu_tuple_cost;
 	path->rows = compressed_path->rows * TARGET_COMPRESSED_BATCH_SIZE;
+	path->total_cost = compressed_path->total_cost + path->rows * cpu_tuple_cost;
 }
 
 /* Smoothstep function S1 (the h01 cubic Hermite spline). */
@@ -864,6 +866,22 @@ ts_decompress_chunk_generate_paths(PlannerInfo *root, RelOptInfo *chunk_rel, con
 	{
 		Path *compressed_path = lfirst(lc);
 
+		/* We want to consider startup costs so that IndexScan is preferred to sorted SeqScan when
+		   we may have a chance to use SkipScan. We consider startup costs for LIMIT queries, and
+		   SkipScan is basically a "LIMIT 1" query run "ndistinct" times. At this point we don't
+		   have all information to check if SkipScan can be used, but we can narrow it down.
+		*/
+		if (!chunk_rel->consider_startup && IsA(compressed_path, IndexPath))
+		{
+			/* Candidate for SELECT DISTINCT SkipScan */
+			if (list_length(root->distinct_pathkeys) == 1
+				/* Candidate for DISTINCT aggregate SkipScan */
+				|| (root->numOrderedAggs >= 1 && list_length(root->group_pathkeys) == 1))
+			{
+				chunk_rel->consider_startup = true;
+			}
+		}
+
 		/*
 		 * We skip any BitmapScan parameterized paths here as supporting
 		 * those would require fixing up the internal scan. Since we
@@ -1061,7 +1079,7 @@ ts_decompress_chunk_generate_paths(PlannerInfo *root, RelOptInfo *chunk_rel, con
 					PathKey *pathkey = (PathKey *) lfirst(lc);
 					EquivalenceClass *pathkey_ec = pathkey->pk_eclass;
 
-					Expr *em_expr = find_em_expr_for_rel(pathkey_ec, chunk_rel);
+					Expr *em_expr = ts_find_em_expr_for_rel(pathkey_ec, chunk_rel);
 
 					/* No em expression found for our rel */
 					if (!em_expr)
@@ -2271,7 +2289,7 @@ build_sortinfo(PlannerInfo *root, const Chunk *chunk, RelOptInfo *chunk_rel,
 		Expr *em_expr = NULL;
 		if (!ec->ec_has_volatile)
 		{
-			em_expr = find_em_expr_for_rel(pk->pk_eclass, compression_info->chunk_rel);
+			em_expr = ts_find_em_expr_for_rel(pk->pk_eclass, compression_info->chunk_rel);
 		}
 		chunk_em_exprs = lappend(chunk_em_exprs, em_expr);
 	}
