@@ -97,14 +97,6 @@ Select avg(temperature) over( order by humidity)
 from conditions
  WITH NO DATA;
 
---aggregate without combine function but stable function
-CREATE MATERIALIZED VIEW mat_m1 WITH (timescaledb.continuous, timescaledb.materialized_only=false)
-AS
-Select json_agg(location)
-from conditions
- group by time_bucket('1week', timec) , location WITH NO DATA;
-;
-
 -- using subqueries
 CREATE MATERIALIZED VIEW mat_m1 WITH (timescaledb.continuous, timescaledb.materialized_only=false)
 AS
@@ -203,28 +195,6 @@ Select sum(humidity), avg(temperature::int4)
 from conditions
 group by rollup(time_bucket('1week', timec) , location )  WITH NO DATA;
 
---NO immutable functions -- check all clauses
-CREATE FUNCTION test_stablefunc(int) RETURNS int LANGUAGE 'sql'
-       STABLE AS 'SELECT $1 + 10';
-
-CREATE MATERIALIZED VIEW mat_m1 WITH (timescaledb.continuous, timescaledb.materialized_only=false)
-AS
-Select sum(humidity), max(timec + INTERVAL '1h')
-from conditions
-group by time_bucket('1week', timec) , location   WITH NO DATA;
-
-CREATE MATERIALIZED VIEW mat_m1 WITH (timescaledb.continuous, timescaledb.materialized_only=false)
-AS
-Select sum( test_stablefunc(humidity::int) ), min(location)
-from conditions
-group by time_bucket('1week', timec) WITH NO DATA;
-
-CREATE MATERIALIZED VIEW mat_m1 WITH (timescaledb.continuous, timescaledb.materialized_only=false)
-AS
-Select sum( temperature ), min(location)
-from conditions
-group by time_bucket('1week', timec), test_stablefunc(humidity::int) WITH NO DATA;
-
 -- Should use CREATE MATERIALIZED VIEW to create continuous aggregates
 CREATE VIEW continuous_aggs_errors_tbl1 WITH (timescaledb.continuous, timescaledb.materialized_only=false) AS
 SELECT time_bucket('1 week', timec)
@@ -253,6 +223,8 @@ Select sum( b), min(c)
 from rowsec_tab
 group by time_bucket('1', a) WITH NO DATA;
 
+DROP TABLE rowsec_tab CASCADE;
+
 drop table conditions cascade;
 
 --negative tests for WITH options
@@ -524,6 +496,9 @@ CREATE MATERIALIZED VIEW cagg1 WITH(timescaledb.continuous, timescaledb.material
 --Check error handling for this case
 SELECT compress_chunk(ch) FROM show_chunks('i2980') ch;
 
+DROP TABLE i2980 CASCADE;
+DROP TABLE comp_ht_test CASCADE;
+
 -- cagg on normal view should error out
 CREATE VIEW v1 AS SELECT now() AS time;
 CREATE MATERIALIZED VIEW cagg1 WITH (timescaledb.continuous, timescaledb.materialized_only=false) AS SELECT time_bucket('1h',time) FROM v1 GROUP BY 1;
@@ -534,3 +509,43 @@ CREATE MATERIALIZED VIEW cagg1 WITH (timescaledb.continuous, timescaledb.materia
 
 -- No FROM clause in CAGG definition
 CREATE MATERIALIZED VIEW cagg1 with (timescaledb.continuous, timescaledb.materialized_only=false) AS SELECT 1 GROUP BY 1 WITH NO DATA;
+
+CREATE TABLE "conditions"(
+  time         TIMESTAMP WITH TIME ZONE NOT NULL,
+  device_id    TEXT,
+  temperature  NUMERIC,
+  humidity     NUMERIC
+) WITH (
+  timescaledb.hypertable,
+  timescaledb.partition_column='time',
+  timescaledb.chunk_interval='1 day'
+);
+
+CREATE MATERIALIZED VIEW conditions_by_hour
+WITH (timescaledb.continuous) AS
+SELECT
+  time_bucket(INTERVAL '1 hour', time) AS bucket,
+  device_id,
+  MAX(temperature),
+  MIN(temperature),
+  COUNT(*)
+FROM conditions
+GROUP BY 1, 2
+WITH NO DATA;
+
+-- Should work
+INSERT INTO conditions (time, device_id, temperature, humidity)
+VALUES
+  ('2023-10-01 00:00:00+00', 'device1', 20.5, 30.0);
+
+\c :TEST_DBNAME :ROLE_CLUSTER_SUPERUSER
+\set ON_ERROR_STOP 0
+\set VERBOSITY default
+
+-- Remove dimensions to test cagg trigger for invalid dimension
+TRUNCATE _timescaledb_catalog.dimension CASCADE;
+
+-- Should fail with no valid open dimension
+DELETE FROM conditions WHERE device_id = 'device1';
+
+DROP TABLE conditions cascade;
