@@ -580,14 +580,13 @@ SELECT remove_compression_policy('metrics_cagg');
 SELECT add_compression_policy('metrics_cagg', '8 day'::interval) AS "COMP_JOB" \gset
 
 --verify that jobs were added for the policies ---
-SELECT materialization_hypertable_schema AS "MAT_SCHEMA_NAME",
-       materialization_hypertable_name AS "MAT_TABLE_NAME",
-       materialization_hypertable_schema || '.' || materialization_hypertable_name AS "MAT_NAME"
+SELECT materialization_hypertable_name AS "MAT_TABLE_NAME",
+       view_name AS "VIEW_NAME"
 FROM timescaledb_information.continuous_aggregates
 WHERE view_name = 'metrics_cagg' \gset
 
 SELECT count(*) FROM timescaledb_information.jobs
-WHERE hypertable_name = :'MAT_TABLE_NAME';
+WHERE hypertable_name = :'VIEW_NAME';
 
 --exec the cagg compression job --
 CALL refresh_continuous_aggregate('metrics_cagg', NULL, '2001-02-01 00:00:00+0');
@@ -609,7 +608,7 @@ WHERE hypertable_name = :'MAT_TABLE_NAME' ORDER BY 1;
 DROP MATERIALIZED VIEW metrics_cagg;
 
 SELECT count(*) FROM timescaledb_information.jobs
-WHERE hypertable_name = :'MAT_TABLE_NAME';
+WHERE hypertable_name = :'VIEW_NAME';
 
 -- add test case for issue 4252
 CREATE TABLE IF NOT EXISTS sensor_data(
