@@ -26,6 +26,8 @@ ALTER TABLE test1 set (timescaledb.compress, timescaledb.compress_segmentby = 'b
 
 SELECT count(compress_chunk(ch)) FROM show_chunks('test1') ch;
 
+VACUUM FULL ANALYZE test1;
+
 
 --make sure allowed ddl still work
 ALTER TABLE test1 CLUSTER ON "test1_Time_idx";
@@ -375,6 +377,8 @@ SELECT a.rolname from pg_class c INNER JOIN pg_authid a ON(c.relowner = a.oid) W
 
 SELECT count(decompress_chunk(ch)) FROM show_chunks('test1') ch;
 
+VACUUM FULL ANALYZE test1;
+
 select add_compression_policy('test1', interval '1 day');
 \set ON_ERROR_STOP 0
 ALTER table test1 set (timescaledb.compress='f');
@@ -407,6 +411,8 @@ ALTER TABLE test1 set (timescaledb.compress, timescaledb.compress_segmentby = 'b
 
 SELECT count(compress_chunk(ch)) FROM show_chunks('test1') ch;
 
+VACUUM FULL ANALYZE test1;
+
 DROP TABLE test1 CASCADE;
 DROP TABLESPACE tablespace1;
 
@@ -627,6 +633,8 @@ FROM compression_insert
 WHERE time >= '2000-01-01 0:00:00+0'
 AND time <= '2000-01-05 23:55:00+0';
 
+VACUUM ANALYZE compression_insert;
+
 -- force index scans to check index mapping
 -- this verifies that we are actually using compressed chunk index scans
 -- previously we could not use indexes on uncompressed chunks due to a bug:
@@ -674,6 +682,8 @@ FROM compression_insert
 WHERE time >= '2000-01-07 0:00:00+0'
 AND time <= '2000-01-11 23:55:00+0';
 
+VACUUM ANALYZE compression_insert;
+
 -- force index scans to check index mapping
 SET enable_seqscan = off;
 EXPLAIN (costs off) SELECT device_id, count(*)
@@ -714,6 +724,8 @@ FROM compression_insert
 WHERE time >= '2000-01-15 0:00:00+0'
 AND time <= '2000-01-19 23:55:00+0';
 
+VACUUM ANALYZE compression_insert;
+
 -- force index scans to check index mapping
 SET enable_seqscan = off;
 EXPLAIN (costs off) SELECT device_id, count(*)
@@ -755,6 +767,8 @@ FROM compression_insert
 WHERE time >= '2000-01-22 0:00:00+0'
 AND time <= '2000-01-26 23:55:00+0';
 
+VACUUM ANALYZE compression_insert;
+
 -- force index scans to check index mapping
 SET enable_seqscan = off;
 EXPLAIN (costs off) SELECT device_id, count(*)
@@ -794,6 +808,8 @@ FROM compression_insert
 WHERE time >= '2000-01-28 0:00:00+0'
 AND time <= '2000-02-01 23:55:00+0';
 
+VACUUM ANALYZE compression_insert;
+
 -- force index scans to check index mapping
 SET enable_seqscan = off;
 EXPLAIN (costs off) SELECT device_id, count(*)
@@ -826,34 +842,64 @@ VALUES -- chunk1
   ('2020-01-01 00:00'::timestamptz, 1, 2),
   ('2020-01-01 00:01'::timestamptz, 2, 2),
   ('2020-01-01 00:04'::timestamptz, 1, 2),
+  ('2020-01-01 00:05'::timestamptz, 3, 1),
+  ('2020-01-01 00:06'::timestamptz, 3, 2),
+  ('2020-01-01 00:07'::timestamptz, 3, 3),
+  ('2020-01-01 00:08'::timestamptz, 4, 1),
   -- chunk2
   ('2021-01-01 00:00'::timestamptz, 1, 2),
   ('2021-01-01 00:04'::timestamptz, 1, 2),
+  ('2021-01-01 00:05'::timestamptz, 3, 1),
+  ('2021-01-01 00:06'::timestamptz, 3, 2),
+  ('2021-01-01 00:07'::timestamptz, 3, 3),
   -- chunk3
   ('2022-01-01 00:00'::timestamptz, 1, 2),
-  ('2022-01-01 00:04'::timestamptz, 1, 2);
+  ('2022-01-01 00:04'::timestamptz, 1, 2),
+  ('2022-01-01 00:05'::timestamptz, 3, 1);
 -- enable compression, compress all chunks
-ALTER TABLE test_partials SET (timescaledb.compress);
+ALTER TABLE test_partials SET (timescaledb.compress,
+    timescaledb.compress_segmentby = 'a', timescaledb.compress_orderby = 'time desc');
 SELECT compress_chunk(show_chunks('test_partials'));
 VACUUM ANALYZE test_partials;
+-- Chunks must be different size for plan stability
+select count(*) from test_partials group by tableoid order by count(*) desc;
+
 -- fully compressed
 EXPLAIN (costs off) SELECT * FROM test_partials ORDER BY time;
+
+
 -- test P, F, F
 INSERT INTO test_partials VALUES ('2020-01-01 00:03', 1, 2);
+-- Chunks must be different size for plan stability
+select count(*) from test_partials group by tableoid order by count(*) desc;
+
 EXPLAIN (costs off) SELECT * FROM test_partials ORDER BY time;
 -- verify correct results
 SELECT * FROM test_partials ORDER BY time;
+
+
 -- make second chunk partially compressed
 -- P, P, F
 INSERT INTO test_partials VALUES ('2021-01-01 00:03', 1, 2);
+-- Chunks must be different size for plan stability
+select count(*) from test_partials group by tableoid order by count(*) desc;
+
 EXPLAIN (costs off) SELECT * FROM test_partials ORDER BY time;
 -- verify correct results
 SELECT * FROM test_partials ORDER BY time;
+
+
 -- third chunk partially compressed and add new chunk
 -- P, P, P, U
 INSERT INTO test_partials VALUES ('2022-01-01 00:03', 1, 2);
 INSERT INTO test_partials VALUES ('2023-01-01 00:03', 1, 2);
+INSERT INTO test_partials VALUES ('2023-01-01 00:05', 3, 1);
+-- Chunks must be different size for plan stability
+select count(*) from test_partials group by tableoid order by count(*) desc;
+
 EXPLAIN (costs off) SELECT * FROM test_partials ORDER BY time;
+
+
 -- F, F, P, U
 -- recompress all chunks
 DO $$
@@ -863,6 +909,7 @@ BEGIN
   FOR chunk IN
   SELECT format('%I.%I', schema_name, table_name)::regclass
     FROM _timescaledb_catalog.chunk WHERE status = 9 and compressed_chunk_id IS NOT NULL AND NOT dropped
+    ORDER BY id
   LOOP
     EXECUTE format('select decompress_chunk(''%s'');', chunk::text);
     EXECUTE format('select compress_chunk(''%s'');', chunk::text);
@@ -870,14 +917,26 @@ BEGIN
 END
 $$;
 INSERT INTO test_partials VALUES ('2022-01-01 00:02', 1, 2);
+VACUUM ANALYZE test_partials;
+-- Chunks must be different size for plan stability
+select count(*) from test_partials group by tableoid order by count(*) desc;
+
 EXPLAIN (COSTS OFF) SELECT * FROM test_partials ORDER BY time;
+
 -- F, F, P, F, F
 INSERT INTO test_partials VALUES ('2024-01-01 00:02', 1, 2);
 SELECT compress_chunk(c) FROM show_chunks('test_partials', newer_than => '2022-01-01') c;
+VACUUM ANALYZE test_partials;
+-- Chunks must be different size for plan stability
+select count(*) from test_partials group by tableoid order by count(*) desc;
+
 EXPLAIN (costs off) SELECT * FROM test_partials ORDER BY time;
 -- verify result correctness
 SELECT * FROM test_partials ORDER BY time;
 
+-- Chunks must be different size for plan stability
+select count(*) from test_partials group by tableoid order by count(*) desc;
+
 -- add test for space partioning with partial chunks
 CREATE TABLE space_part (time timestamptz, a int, b int, c int);
 SELECT create_hypertable('space_part', 'time', chunk_time_interval => INTERVAL '1 day');
@@ -902,6 +961,7 @@ INSERT INTO space_part VALUES
 -- chunk1
 ('2020-01-01 00:01', 1, 1, 1),
 ('2020-01-01 00:01', 2, 1, 1);
+VACUUM ANALYZE space_part;
 
 -------- now enable the space partitioning, this will take effect for chunks created subsequently
 SELECT add_dimension('space_part', 'a', number_partitions => 5);
@@ -919,6 +979,7 @@ INSERT INTO space_part VALUES
 EXPLAIN (COSTS OFF) SELECT * FROM space_part ORDER BY time;
 -- compress them
 SELECT compress_chunk(c, if_not_compressed=>true) FROM show_chunks('space_part') c;
+VACUUM ANALYZE space_part;
 -- plan still ok
 EXPLAIN (COSTS OFF) SELECT * FROM space_part ORDER BY time;
 -- make second one of them partial
@@ -951,6 +1012,7 @@ values ('meter1', 1, 2.3, '2022-01-01'::timestamptz, '2022-01-01'::timestamptz),
 select compress_chunk(show_chunks('mytab'));
 REINDEX TABLE mytab; -- should update index
 select decompress_chunk(show_chunks('mytab'));
+vacuum analyze mytab;
 \set EXPLAIN 'EXPLAIN (costs off,timing off,summary off)'
 \set EXPLAIN_ANALYZE 'EXPLAIN (analyze,costs off,timing off,summary off)'
 -- do index scan on uncompressed, should give correct results
