@@ -775,3 +775,227 @@ SELECT count(*) FROM unique_null;
 
 DROP TABLE unique_null;
 
+-- test insert with UNIQUE constraint and all compression algorithms
+set timescaledb.enable_bool_compression = on;
+
+CREATE TABLE unique_all(
+	time timestamptz NOT NULL,
+	device_id int,
+	bool_type bool,
+	numeric_type numeric,
+	float_type float,
+	text_type text,
+	bigint_type bigint,
+	jsonb_type jsonb
+);
+-- Since we don't test uniqueness when we detect a NULL value, making the constraint
+-- with NULLS NOT DISTINCT
+CREATE UNIQUE INDEX ultimate_unique ON unique_all(time, device_id, bool_type, numeric_type, float_type, text_type, bigint_type, jsonb_type) NULLS NOT DISTINCT;
+SELECT table_name FROM create_hypertable('unique_all', 'time');
+ALTER TABLE unique_all SET (tsdb.compress, tsdb.compress_segmentby = 'device_id', tsdb.compress_orderby = 'time');
+
+INSERT INTO unique_all VALUES('2024-01-01 00:00', 1, true, 1.0, 1.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:01', 1, true, 1.0, 1.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:02', 1, false, 1.0, 1.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:03', 1, true, 2.0, 1.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:04', 1, true, 1.0, 2.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:05', 1, true, 1.0, 1.0, 'second', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:06', 1, true, 1.0, 1.0, 'first', 2, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:07', 1, true, 1.0, 1.0, 'first', 1, '{"second":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:08', 1, true, 1.0, 1.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:09', 1, NULL, 1.0, 1.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:10', 1, true, NULL, 1.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:11', 1, true, 1.0, NULL, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:12', 1, true, 1.0, 1.0, NULL, 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:13', 1, true, 1.0, 1.0, 'first', NULL, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:14', 1, true, 1.0, 1.0, 'first', 1, NULL);
+SELECT count(compress_chunk(c)) FROM show_chunks('unique_all') c;
+
+-- test duplicates
+\set ON_ERROR_STOP 0
+INSERT INTO unique_all VALUES('2024-01-01 00:00', 1, true, 1.0, 1.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:01', 1, true, 1.0, 1.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:02', 1, false, 1.0, 1.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:03', 1, true, 2.0, 1.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:04', 1, true, 1.0, 2.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:05', 1, true, 1.0, 1.0, 'second', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:06', 1, true, 1.0, 1.0, 'first', 2, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:07', 1, true, 1.0, 1.0, 'first', 1, '{"second":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:08', 1, true, 1.0, 1.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:09', 1, NULL, 1.0, 1.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:10', 1, true, NULL, 1.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:11', 1, true, 1.0, NULL, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:12', 1, true, 1.0, 1.0, NULL, 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:13', 1, true, 1.0, 1.0, 'first', NULL, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:14', 1, true, 1.0, 1.0, 'first', 1, NULL);
+\set ON_ERROR_STOP 1
+
+-- test non-duplicates
+INSERT INTO unique_all VALUES('2024-01-01 00:01:01', 1, true, 1.0, 1.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:02', 1, true, 1.0, 1.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:03', 1, true, 3.0, 1.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:04', 1, true, 1.0, 3.0, 'first', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:05', 1, true, 1.0, 1.0, 'third', 1, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:06', 1, true, 1.0, 1.0, 'first', 3, '{"first":true}');
+INSERT INTO unique_all VALUES('2024-01-01 00:07', 1, true, 1.0, 1.0, 'first', 1, '{"third":true}');
+
+-- test non-duplicates with non-NULL default values
+ALTER TABLE unique_all ADD COLUMN added_non_null int DEFAULT 4;
+
+DROP INDEX ultimate_unique;
+CREATE UNIQUE INDEX ultimate_unique ON unique_all(time, device_id, bool_type, numeric_type, float_type, text_type, bigint_type, jsonb_type, added_non_null) NULLS NOT DISTINCT;
+
+\set ON_ERROR_STOP 0
+INSERT INTO unique_all VALUES('2024-01-01 00:00', 1, true, 1.0, 1.0, 'first', 1, '{"first":true}', 4);
+\set ON_ERROR_STOP 1
+INSERT INTO unique_all VALUES('2024-01-01 00:00', 1, true, 1.0, 1.0, 'first', 1, '{"first":true}', 5);
+
+-- make sure everything is correct after recompressing the chunk
+SELECT count(decompress_chunk(c)) FROM show_chunks('unique_all') c;
+SELECT count(compress_chunk(c)) FROM show_chunks('unique_all') c;
+
+\set ON_ERROR_STOP 0
+INSERT INTO unique_all VALUES('2024-01-01 00:00', 1, true, 1.0, 1.0, 'first', 1, '{"first":true}', 4);
+\set ON_ERROR_STOP 1
+
+-- test non-duplicates with NULL default values
+ALTER TABLE unique_all ADD COLUMN added_null int DEFAULT NULL;
+DROP INDEX ultimate_unique;
+CREATE UNIQUE INDEX ultimate_unique ON unique_all(time, device_id, bool_type, numeric_type, float_type, text_type, bigint_type, jsonb_type, added_non_null, added_null) NULLS NOT DISTINCT;
+
+\set ON_ERROR_STOP 0
+INSERT INTO unique_all VALUES('2024-01-01 00:00', 1, true, 1.0, 1.0, 'first', 1, '{"first":true}', 4, NULL);
+INSERT INTO unique_all VALUES('2024-01-01 00:00', 1, true, 1.0, 1.0, 'first', 1, '{"first":true}', 5, NULL);
+\set ON_ERROR_STOP 1
+
+INSERT INTO unique_all VALUES('2024-01-01 00:00', 1, true, 1.0, 1.0, 'first', 1, '{"first":true}', 6, NULL);
+
+-- make sure everything is correct after recompressing the chunk
+SELECT count(decompress_chunk(c)) FROM show_chunks('unique_all') c;
+SELECT count(compress_chunk(c)) FROM show_chunks('unique_all') c;
+
+\set ON_ERROR_STOP 0
+INSERT INTO unique_all VALUES('2024-01-01 00:00', 1, true, 1.0, 1.0, 'first', 1, '{"first":true}', 4, NULL);
+INSERT INTO unique_all VALUES('2024-01-01 00:00', 1, true, 1.0, 1.0, 'first', 1, '{"first":true}', 5, NULL);
+\set ON_ERROR_STOP 1
+
+DROP TABLE unique_all;
+
+-- test NULL compression algorithm
+
+CREATE TABLE unique_null(
+	time timestamptz NOT NULL,
+	device_id int,
+	tag text,
+	tag2 text,
+	value float
+);
+CREATE UNIQUE INDEX unique_nulls ON unique_null(time, device_id, tag, tag2) NULLS NOT DISTINCT;
+SELECT table_name FROM create_hypertable('unique_null', 'time');
+ALTER TABLE unique_null SET (tsdb.compress, tsdb.compress_segmentby = 'device_id', tsdb.compress_orderby = 'time');
+INSERT INTO unique_null VALUES
+('2024-01-01 00:00', 1, NULL, '1', 1),
+('2024-01-01 00:00', 1, NULL, '2', 2),
+('2024-01-01 00:00', 1, NULL, '3', 3);
+SELECT count(compress_chunk(c)) FROM show_chunks('unique_null') c;
+\set ON_ERROR_STOP 0
+INSERT INTO unique_null VALUES ('2024-01-01 00:00', 1, NULL, '1', 1);
+\set ON_ERROR_STOP 1
+
+ALTER TABLE unique_null ADD COLUMN tag3 text DEFAULT 'default value';
+DROP INDEX unique_nulls;
+CREATE UNIQUE INDEX unique_nulls ON unique_null(time, device_id, tag, tag2, tag3) NULLS NOT DISTINCT;
+
+\set ON_ERROR_STOP 0
+INSERT INTO unique_null VALUES ('2024-01-01 00:00', 1, NULL, '1', 1, 'default value');
+\set ON_ERROR_STOP 1
+
+DROP TABLE unique_null;
+
+-- test NUMERIC data type
+
+CREATE TABLE unique_numeric(
+	time timestamptz NOT NULL,
+	device_id numeric(6,2) NOT NULL,
+	tag text,
+	tag2 text,
+	value float
+);
+CREATE UNIQUE INDEX unique_numeric_idx ON unique_numeric(time, device_id, tag, tag2);
+SELECT table_name FROM create_hypertable('unique_numeric', 'time');
+ALTER TABLE unique_numeric SET (tsdb.compress, tsdb.compress_segmentby = 'device_id', tsdb.compress_orderby = 'time');
+INSERT INTO unique_numeric VALUES
+('2024-01-01 00:00', 1.1, NULL, '1', 1),
+('2024-01-01 00:00', 1.2, NULL, '2', 2),
+('2024-01-01 00:00', 1.3, NULL, '3', 3);
+SELECT count(compress_chunk(c)) FROM show_chunks('unique_numeric') c;
+INSERT INTO unique_numeric VALUES ('2024-01-01 00:00', 1.4, NULL, '1', 1);
+
+DROP TABLE unique_numeric;
+
+-- test UUID data type
+
+CREATE TABLE unique_uuid(
+	time timestamptz NOT NULL,
+	device_id UUID NOT NULL,
+	tag text,
+	tag2 text,
+	value float
+);
+CREATE UNIQUE INDEX unique_uuid_idx ON unique_uuid(time, device_id, tag, tag2);
+SELECT table_name FROM create_hypertable('unique_uuid', 'time');
+ALTER TABLE unique_uuid SET (tsdb.compress, tsdb.compress_segmentby = 'device_id', tsdb.compress_orderby = 'time');
+INSERT INTO unique_uuid VALUES
+('2024-01-01 00:00', '018a0d1a-8e6a-7000-8000-000000000000', NULL, '1', 1),
+('2024-01-01 00:00', '018a0d1a-8e6a-7000-8000-000000000000', NULL, '1', 2),
+('2024-01-01 00:00', '018a0d1a-8e6a-7000-8000-000000000001', NULL, '1', 1),
+('2024-01-01 00:00', '018a0d1a-8e6a-7000-8000-000000000001', NULL, '1', 2);
+SELECT count(compress_chunk(c)) FROM show_chunks('unique_uuid') c;
+INSERT INTO unique_uuid VALUES ('2024-01-01 00:00', '018a0d1a-8e6a-7000-8000-000000000001', NULL, '1', 3);
+
+DROP TABLE unique_uuid;
+
+-- regression test for SDC 3006
+
+CREATE TABLE t_1sec(
+	"timestamp"                         timestamp with time zone NOT NULL,
+	device_id                           text,
+	speed                               double precision,
+	derived                             boolean
+);
+
+ALTER TABLE t_1sec ADD CONSTRAINT unique_device_timestamp_sensor_derived
+	UNIQUE (device_id, "timestamp", derived);
+
+CREATE INDEX t_1sec_timestamp_idx
+	ON t_1sec
+	USING btree ("timestamp" DESC);
+
+SELECT create_hypertable(
+	relation => 't_1sec',
+	time_column_name => 'timestamp',
+	chunk_time_interval => interval '06:00:00',
+	create_default_indexes => false
+);
+
+ALTER TABLE t_1sec SET (
+	timescaledb.compress,
+	timescaledb.compress_segmentby = 'device_id',
+	timescaledb.compress_orderby='"timestamp"'
+);
+
+insert into t_1sec ( "timestamp", device_id, speed, derived) values
+( '2025-06-06 10:00:00', 'device_id_1', 100, true),
+( '2025-06-06 10:01:00', 'device_id_2', 100, true),
+( '2025-06-06 10:02:00', 'device_id_3', 100, true);
+
+SELECT compress_chunk(show_chunks('t_1sec'));
+
+insert into t_1sec ( "timestamp", device_id, speed, derived) values
+( '2025-06-06 10:03:00', 'device_id_1', 100, true),
+( '2025-06-06 10:00:00', 'device_id_1', 110, true),
+( '2025-06-06 10:00:00', 'device_id_1', 110, false)
+ON CONFLICT DO NOTHING;
+
+DROP TABLE t_1sec;
+
