@@ -5,6 +5,10 @@
 -- Avoid chunkwise aggregation to make the test stable
 set timescaledb.enable_chunkwise_aggregation to off;
 
+-- Hypercore TAM uses alternative sparse index predicate pushdown code which
+-- lacks the support for the bloom1 sparse index at the moment.
+set timescaledb.enable_sparse_index_bloom to off;
+
 create table readings(time timestamptz,
        location text,
        device int,
@@ -190,7 +194,7 @@ select count(*) from :chunk where location = 1::text;
 -- ColumnarScan declares itself as projection capable. This query
 -- would add a Result node on top if ColumnarScan couldn't project.
 set timescaledb.enable_columnarscan=true;
-explain
+explain (costs off)
 select time, device+device as device_x2 from :chunk limit 1;
 select time, device+device as device_x2 from :chunk limit 1;
 
@@ -496,3 +500,6 @@ fetch backward 2 from cur1;
 fetch backward 2 from cur1;
 close cur1;
 commit;
+
+reset timescaledb.enable_chunkwise_aggregation;
+reset timescaledb.enable_sparse_index_bloom;
